from tqdm import tqdm, trange
from pathlib import Path
from os.path import join, getsize
from joblib import Parallel, delayed
from torch.utils.data import Dataset


class SnipsDataset(Dataset):
    def __init__(self, split, tokenizer, bucket_size, path, num_workers=12, ascending=False, **kwargs):
        # Setup
        self.path = path
        self.bucket_size = bucket_size
        self.speaker_list = kwargs[f'{split}_speakers'] if type(split) == str else kwargs[f'{split[0]}_speakers']

        # Load transcription
        transcripts_file = open(join(self.path, 'all.iob.snips.txt' if '-slot' in tokenizer.token_type else 'all-trans.txt')).readlines()
        transcripts = {}
        for line in transcripts_file:
            line = line.strip().split(' ')
            index = line[0]
            sent = ' '.join(line[1:])
            transcripts[index] = sent

        # List all wave files
        file_list = []
        for s in split:
            split_list = list(Path(join(path, s)).rglob("*.wav"))
            #for spk in self.speaker_list:
            #    print('- '+spk)
            #    for wav_file in tmp_split_list:
            #        if spk in str(wav_file):
            #            split_list.append(wav_file)
            new_list = []
            uf = 0
            for i in trange(len(split_list), desc='checking files'):
                uid = str(split_list[i]).split('/')[-1].split('.wav', 1)[0].split('/')[-1]
                if uid in transcripts:
                    for spk in self.speaker_list:
                        if uid[:len(spk)] == spk:
                            new_list.append(split_list[i])
                            break
                else:
                    print(split_list[i], "Not Found")
                    uf += 1
            print("%d wav file with label not found in text file!"%uf)
            split_list = new_list
            print(f'loaded audio from {len(self.speaker_list)} speakers {str(self.speaker_list)} with {len(split_list)} examples.')
            assert len(split_list) > 0, "No data found @ {}".format(join(path,s))
            file_list += split_list
        # Read text
        #text = Parallel(n_jobs=num_workers)(
        #    delayed(read_text)(str(f)) for f in file_list)
        #text = Parallel(n_jobs=-1)(delayed(tokenizer.encode)(txt) for txt in text)
        text = [transcripts[str(f).split('.wav', 1)[0].split('/')[-1]] for f in file_list]
        text = [tokenizer.encode(txt) for txt in tqdm(text, desc='tokenizing')]

        # Sort dataset by text length
        #file_len = Parallel(n_jobs=num_workers)(delayed(getsize)(f) for f in file_list)
        self.file_list, self.text = zip(*[(f_name, txt)
                                          for f_name, txt in sorted(zip(file_list, text), reverse=not ascending, key=lambda x:len(x[1]))])

    def __getitem__(self, index):
        if self.bucket_size > 1:
            # Return a bucket
            index = min(len(self.file_list)-self.bucket_size, index)
            return [(f_path, txt) for f_path, txt in
                    zip(self.file_list[index:index+self.bucket_size], self.text[index:index+self.bucket_size])]
        else:
            return self.file_list[index], self.text[index]

    def __len__(self):
        return len(self.file_list)



from collections import Counter, OrderedDict

from s3prl.dataset.base import AugmentedDynamicItemDataset
from s3prl.dataio.sampler import BalancedWeightedSampler


def test_balanced_weighted_sampler():
    dataset = AugmentedDynamicItemDataset(
        OrderedDict(
            {
                0: dict(label="a"),
                1: dict(label="a"),
                2: dict(label="b"),
                3: dict(label="a"),
            }
        )
    )
    batch_size = 5
    prev_diff_ratio = 1.0
    for duplicate in [10000, 100000]:
        sampler = BalancedWeightedSampler(
            dataset, batch_size=batch_size, duplicate=duplicate, seed=0
        )
        indices = list(sampler)
        assert len(indices[0]) == batch_size

        counter = Counter()
        for batch_indices in indices:
            for idx in batch_indices:
                counter.update(dataset[idx]["label"])

        diff_ratio = abs(counter["a"] - counter["b"]) / duplicate * len(dataset)
        assert diff_ratio < prev_diff_ratio
        prev_diff_ratio = diff_ratio

    diff_ratio < 0.05



# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ generate_len_for_bucket.py ]
#   Synopsis     [ preprocess audio speech to generate meta data for dataloader bucketing ]
#   Author       [ Andy T. Liu (Andi611) ]
#   Copyright    [ Copyleft(c), Speech Lab, NTU, Taiwan ]
#   Reference    [ https://github.com/Alexander-H-Liu/End-to-end-ASR-Pytorch ]
"""*********************************************************************************************"""


###############
# IMPORTATION #
###############
import os
import argparse
import torchaudio
import numpy as np
import pandas as pd
from tqdm import tqdm
from pathlib import Path
from joblib import Parallel, delayed


#############################
# PREPROCESS CONFIGURATIONS #
#############################
def get_preprocess_args():
    
    parser = argparse.ArgumentParser(description='preprocess arguments for any dataset.')

    parser.add_argument('-i', '--input_data', default='../LibriSpeech/', type=str, help='Path to your LibriSpeech directory', required=False)
    parser.add_argument('-o', '--output_path', default='./data/', type=str, help='Path to store output', required=False)
    parser.add_argument('-a', '--audio_extension', default='.flac', type=str, help='audio file type (.wav / .flac / .mp3 / etc)', required=False)
    parser.add_argument('-n', '--name', default='len_for_bucket', type=str, help='Name of the output directory', required=False)
    parser.add_argument('--n_jobs', default=-1, type=int, help='Number of jobs used for feature extraction', required=False)

    args = parser.parse_args()
    return args


##################
# EXTRACT LENGTH #
##################
def extract_length(input_file):
    torchaudio.set_audio_backend("sox_io")
    return torchaudio.info(input_file).num_frames


###################
# GENERATE LENGTH #
###################
def generate_length(args, tr_set, audio_extension):
    
    for i, s in enumerate(tr_set):
        if os.path.isdir(os.path.join(args.input_data, s.lower())):
            s = s.lower()
        elif os.path.isdir(os.path.join(args.input_data, s.upper())):
            s = s.upper()
        else:
            assert NotImplementedError

        print('')
        todo = list(Path(os.path.join(args.input_data, s)).rglob('*' + audio_extension)) # '*.flac'
        print(f'Preprocessing data in: {s}, {len(todo)} audio files found.')

        output_dir = os.path.join(args.output_path, args.name)
        if not os.path.exists(output_dir): os.makedirs(output_dir)

        print('Extracting audio length...', flush=True)
        tr_x = Parallel(n_jobs=args.n_jobs)(delayed(extract_length)(str(file)) for file in tqdm(todo))

        # sort by len
        sorted_todo = [os.path.join(s, str(todo[idx]).split(s+'/')[-1]) for idx in reversed(np.argsort(tr_x))]
        # Dump data
        df = pd.DataFrame(data={'file_path':[fp for fp in sorted_todo], 'length':list(reversed(sorted(tr_x))), 'label':None})
        df.to_csv(os.path.join(output_dir, tr_set[i] + '.csv'))

    print('All done, saved at', output_dir, 'exit.')


########
# MAIN #
########
def main():

    # get arguments
    args = get_preprocess_args()
    
    if 'librilight' in args.input_data.lower():
        SETS = ['small', 'medium', 'large'] + ['small-splitted', 'medium-splitted', 'large-splitted']
    elif 'librispeech' in args.input_data.lower():
        SETS = ['train-clean-100', 'train-clean-360', 'train-other-500', 'dev-clean', 'dev-other', 'test-clean', 'test-other']
    elif 'timit' in args.input_data.lower():
        SETS = ['TRAIN', 'TEST']
    else:
        raise NotImplementedError
    # change the SETS list to match your dataset, for example:
    # SETS = ['train', 'dev', 'test']
    # SETS = ['TRAIN', 'TEST']
    # SETS = ['train-clean-100', 'train-clean-360', 'train-other-500', 'dev-clean', 'dev-other', 'test-clean', 'test-other']
    
    # Select data sets
    for idx, s in enumerate(SETS):
        print('\t', idx, ':', s)
    tr_set = input('Please enter the index of splits you wish to use preprocess. (seperate with space): ')
    tr_set = [SETS[int(t)] for t in tr_set.split(' ')]

    # Acoustic Feature Extraction & Make Data Table
    generate_length(args, tr_set, args.audio_extension)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ ark2libri.py ]
#   Synopsis     [ process the .ark file preprocessed by kaldi for our dataloader ]
#   Author       [ Andy T. Liu (Andi611) ]
#   Copyright    [ Copyleft(c), Speech Lab, NTU, Taiwan ]
#   Reference    [ https://github.com/nttcslab-sp/kaldiio ]
"""*********************************************************************************************"""

###############
# IMPORTATION #
###############
import os
import pickle
import operator
import numpy as np
import pandas as pd
from tqdm import tqdm
from kaldiio import ReadHelper


############
# SETTINGS #
############
DATA_TYPE = 'fmllr' # this can be either ['mfcc', 'fbank', 'fmllr']
KALDI_ROOT = '/media/andi611/1TBSSD/kaldi/' # change this to your own kaldi root
LIBRI_PATH = os.path.join(KALDI_ROOT, 'egs/librispeech/s5/' + DATA_TYPE + '_cmvn/') # this needs to be generated by the scripts `dump_fmllr_cmvn.sh` or `dump_mfcc_cmvn.sh`
OUTPUT_DIR = '../data/libri_' + DATA_TYPE + '_cmvn' # 


############
# CONSTANT #
############
SETS = ['dev_clean', 'test_clean', 'train_clean_100', 'train_clean_360', 'train_other_500'] # you should not need to change this


########
# MAIN #
########
def main():
    if not os.path.isdir(KALDI_ROOT):
        print('CHANGE THIS TO YOUR OWN KALDI ROOT: ', KALDI_ROOT)
        exit()

    if not os.path.isdir(LIBRI_PATH):
        print('Invalid path for the kaldi librispeech dataset: ', LIBRI_PATH)
        print('Please run the kaldi scripts first! More information are described in the README file and Wiki page.')

    if not os.path.isdir(OUTPUT_DIR):
        os.mkdir(OUTPUT_DIR)

    # read data from the preprocessed kaldi directory
    for s in SETS:
        with ReadHelper('ark:' + LIBRI_PATH + s + '/' + DATA_TYPE + '_cmvn.ark') as reader:
            
            output = {}
            print('Preprocessing', s, 'data...')

            cur_dir = os.path.join(OUTPUT_DIR, s.replace('_', '-'))
            if not os.path.isdir(cur_dir): os.mkdir(cur_dir)

            for key, array in tqdm(reader):

                array = np.asarray(array).astype('float32')
                np.save(os.path.join(cur_dir, key), array)
                output[os.path.join(s.replace('_', '-'), key + '.npy')] = len(array)

            output = sorted(output.items(), key=operator.itemgetter(1), reverse=True)
            df = pd.DataFrame(data={'file_path':[fp for fp, l in output], 'length':[l for fp, l in output], 'label':'None'})
            df.to_csv(os.path.join(OUTPUT_DIR, s.replace('_', '-') + '.csv'))

    print('[ARK-TO-LIBRI] - All done, saved at \'' + str(OUTPUT_DIR) + '\', exit.')
    exit()

if __name__ == '__main__':
    main()
    



# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ preprocess_any.py ]
#   Synopsis     [ preprocess text transcripts and audio speech for any dataset ]
#   Author       [ Andy T. Liu (Andi611) ]
#   Copyright    [ Copyleft(c), Speech Lab, NTU, Taiwan ]
#   Reference    [ https://github.com/Alexander-H-Liu/End-to-end-ASR-Pytorch ]
"""*********************************************************************************************"""


###############
# IMPORTATION #
###############
import os
import sys
import pickle
import argparse
import numpy as np
import pandas as pd
from tqdm import tqdm
from pathlib import Path
from joblib import Parallel, delayed
from utility.audio import extract_feature, num_mels, num_mfcc, num_freq


SETS = ['train', 'dev', 'test'] # change these to match your dataset
# SETS = ['train-clean-100', 'train-clean-360', 'train-other-500', 'dev-clean', 'dev-other', 'test-clean', 'test-other']#


##################
# BOOLEAB STRING #
##################
def boolean_string(s):
    if s not in ['False', 'True']:
        raise ValueError('Not a valid boolean string')
    return s == 'True'


#############################
# PREPROCESS CONFIGURATIONS #
#############################
def get_preprocess_args():
    
    parser = argparse.ArgumentParser(description='preprocess arguments for any dataset.')

    parser.add_argument('--output_path', default='./data/', type=str, help='Path to store output', required=False)
    parser.add_argument('--audio_extention', default='.flac', type=str, help='audio file type (.wav / .flac / .mp3 / etc)', required=False)

    parser.add_argument('--feature_type', default='fbank', type=str, help='Feature type ( mfcc / fbank / mel / linear )', required=False)
    parser.add_argument('--delta', default=False, type=boolean_string, help='Append Delta', required=False)
    parser.add_argument('--delta_delta', default=False, type=boolean_string, help='Append Delta Delta', required=False)
    parser.add_argument('--apply_cmvn', default=True, type=boolean_string, help='Apply CMVN on feature', required=False)

    parser.add_argument('--n_jobs', default=-1, type=int, help='Number of jobs used for feature extraction', required=False)
    parser.add_argument('--name', default='None', type=str, help='Name of the output directory', required=False)

    args = parser.parse_args()
    return args


#######################
# ACOUSTIC PREPROCESS #
#######################
def acoustic_preprocess(args, tr_set, dim, audio_extention):
    
    for i, s in enumerate(tr_set):
        print('')
        print('Preprocessing data in: ', s, end='')
        todo = list(Path(os.path.join(args.data_root, s)).rglob('*' + audio_extention)) # '*.flac'
        print(len(todo), 'audio files found.')

        if args.name == 'None':
            output_dir = os.path.join(args.output_path, '_'.join(['NewData', str(args.feature_type)+str(dim)]))
        else:
            output_dir = os.path.join(args.output_path, args.name)
        if not os.path.exists(output_dir): os.makedirs(output_dir)
        cur_path = os.path.join(output_dir, tr_set[i])
        if not os.path.exists(cur_path): os.makedirs(cur_path)

        print('Extracting acoustic feature...', flush=True)
        tr_x = Parallel(n_jobs=args.n_jobs)(delayed(extract_feature)(str(file), feature=args.feature_type, \
                                            delta=args.delta, delta_delta=args.delta_delta, cmvn=args.apply_cmvn, \
                                            save_feature=os.path.join(cur_path, str(file).split('/')[-1].replace(audio_extention, ''))) for file in tqdm(todo))

        # sort by len
        sorted_todo = [os.path.join(tr_set[i], str(todo[idx]).split('/')[-1].replace(audio_extention, '.npy')) for idx in reversed(np.argsort(tr_x))]
        # Dump data
        df = pd.DataFrame(data={'file_path':[fp for fp in sorted_todo], 'length':list(reversed(sorted(tr_x))), 'label':None})
        df.to_csv(os.path.join(output_dir, tr_set[i] + '.csv'))

    print('All done, saved at', output_dir, 'exit.')


########
# MAIN #
########
def main():

    # get arguments
    args = get_preprocess_args()
    mel_dim = num_mels * (1 + int(args.delta) + int(args.delta_delta))
    mfcc_dim = num_mfcc * (1 + int(args.delta) + int(args.delta_delta))
    dim = num_freq if args.feature_type == 'linear' else (mfcc_dim if args.feature_type == 'mfcc' else mel_dim)
    print('Delta: ', args.delta, '. Delta Delta: ', args.delta_delta, '. Cmvn: ', args.apply_cmvn)

    # Select data sets
    for idx, s in enumerate(SETS):
        print('\t', idx, ':', s)
    tr_set = input('Please enter the index of splits you wish to use preprocess. (seperate with space): ')
    tr_set = [SETS[int(t)] for t in tr_set.split(' ')]

    # Acoustic Feature Extraction & Make Data Table
    acoustic_preprocess(args, tr_set, dim, args.audio_extention)


if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ preprocess_mosei.py ]
#   Synopsis     [ preprocessing for the MOSEI dataset ]
#   Author       [ Andy T. Liu (Andi611) ]
#   Copyright    [ Copyleft(c), Speech Lab, NTU, Taiwan ]
"""*********************************************************************************************"""


###############
# IMPORTATION #
###############
import os
import shutil
import glob
import sys
import pickle
import argparse
import numpy as np
import pandas as pd
from tqdm import tqdm
from pathlib import Path
import shutil
from joblib import Parallel, delayed
from utility.asr import encode_target
from utility.audio import extract_feature, mel_dim, num_freq


def boolean_string(s):
    if s not in ['False', 'True']:
        raise ValueError('Not a valid boolean string')
    return s == 'True'

def bracket_underscore(string):
    split = string.split('[')
    utterance_name = split[0]
    number = int(split[1].split(']')[0])
    string = utterance_name + '_' + str(number + 1)
    return string

def underscore_bracket(string):
    split = string.split('_')
    number = int(split[-1][:-4])
    utterance_name = '_'.join(split[:-1])
    string = utterance_name + '[' + str(number - 1) + ']'
    return string


def get_preprocess_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--flac_path', default='../../data/mosei/flac', type=str, help='Path to MOSEI segmented FLAC files')
    parser.add_argument('--output_path', default='../../data/mosei', type=str, help='Path to store segmented npys', required=False)
    parser.add_argument('--feature_type', default='mel', type=str, help='Feature type ( mfcc / fbank / mel / linear )', required=False)
    parser.add_argument('--apply_cmvn', default=True, type=boolean_string, help='Apply CMVN on feature', required=False)
    parser.add_argument('--n_jobs', default=-1, type=int, help='Number of jobs used for feature extraction', required=False)
    args = parser.parse_args()
    return args


def extract_mosei(args, dim):
    assert os.path.exists(args.flac_path), f'{args.flac_path} not exists'
    todo = list(Path(args.flac_path).glob("*.flac"))
    print(len(todo),'audio files found in MOSEI')

    assert args.feature_type in ['mel', 'linear', 'fbank'], 'Feature type unsupported'

    if not os.path.exists(args.output_path):
        os.makedirs(args.output_path)
    npy_dir = os.path.join(args.output_path,str(args.feature_type)+str(dim))
    for target_dir in [npy_dir]:
        if os.path.exists(target_dir):
            decision = input(f'{target_dir} already exists. Remove it? [Y/N]: ')
            if decision.upper() == 'Y':
                print(f'Removing {target_dir}')
                shutil.rmtree(target_dir)
            else:
                print('Abort')
                exit(0)
        os.makedirs(target_dir)

    print('Extracting acoustic feature...',flush=True)
    tr_x = Parallel(n_jobs=args.n_jobs)(delayed(extract_feature)(str(file), feature=args.feature_type, cmvn=args.apply_cmvn, \
                                        save_feature=os.path.join(npy_dir, str(file).split('/')[-1].replace('.flac',''))) for file in tqdm(todo))


########
# MAIN #
########
def main():
    # get arguments
    args = get_preprocess_args()
    dim = num_freq if args.feature_type == 'linear' else mel_dim

    # Acoustic Feature Extraction
    extract_mosei(args, dim)


if __name__ == '__main__':
    main()




# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ timit2ark.py ]
#   Synopsis     [ plug-in our own preprocessed features to .ark files for kaldi ]
#   Author       [ Andy T. Liu (Andi611) ]
#   Copyright    [ Copyleft(c), Speech Lab, NTU, Taiwan ]
#   Reference    [ https://github.com/mravanelli/pytorch-kaldi#how-can-i-use-my-own-dataset ]
"""*********************************************************************************************"""

###############
# IMPORTATION #
###############
import os
import pickle
import kaldi_io
import numpy as np
from tqdm import tqdm


############
# SETTINGS #
############
KALDI_ROOT = '/media/andi611/1TBSSD/kaldi/' # !!!!!!!!!!!!!!!!! CHANGE THIS TO YOUR OWN KALDI ROOT !!!!!!!!!!!!!!!!! #
INPUT_PATH = '../data/timit_mel160_phoneme63' # this can be generated with 'preprocess_timit.py'

INPUT_SETS = ['train', 'test'] # you should not need to change this
OUTPUT_SETS = ['train', 'dev', 'test'] # you should not need to change this

TIMIT_PATH = os.path.join(KALDI_ROOT, 'egs/timit/s5/') # you should not need to change this
SOURCE_DIR = os.path.join(TIMIT_PATH, 'data-kaldi-mel') # the data directory generated by kaldi script
OUTPUT_PATH = os.path.join(TIMIT_PATH, 'timit_mel160_arked') # you should not need to change this


########
# MAIN #
########
def main():
    if not os.path.isdir(KALDI_ROOT):
        print('CHANGE THIS TO YOUR OWN KALDI ROOT: ', KALDI_ROOT)
        exit()

    if not os.path.isdir(INPUT_PATH):
        print('Invalid path for the preprocessed timit dataset: ', INPUT_PATH)
        print('Please run \'preprocess_timit.py\' first!')
        exit()
        
    if not os.path.isdir(SOURCE_DIR):
        print('Invalid path for the source directory: ', SOURCE_DIR)
        print('Please read the Wiki page for instructions!')
        exit()

    if not os.path.isdir(OUTPUT_PATH):
        os.mkdir(OUTPUT_PATH)

    # read train and test from the preprocessed directory
    x, ids = [], []
    for s in INPUT_SETS:
        with open(os.path.join(INPUT_PATH, s + '_x.pkl'), 'rb') as fp:
            x += pickle.load(fp)
        with open(os.path.join(INPUT_PATH, s + '_id.pkl'), 'rb') as fp:
            ids += pickle.load(fp)
        assert len(x)==len(ids)
    print('[TIMIT-to-ARK] - ', 'Total Dataset len:', len(x))

    # construct all input dict
    all_inputs = {}
    for idx, i in enumerate(ids):
        i = str(i).strip('.wav').split('/')
        i = i[-2].upper() + '_' + i[-1].upper()
        all_inputs[i] = np.asarray(x[idx])

    # filter all input with kaldi generated files
    for s in OUTPUT_SETS:
        if not os.path.isdir(SOURCE_DIR):
            raise NotADirectoryError('Source directory does not exist!', SOURCE_DIR)    
        
        if not os.path.isdir(OUTPUT_PATH + '/' + str(s)):
            os.mkdir(OUTPUT_PATH + '/' + str(s))
        
        # read train / dev / test from the kaldi generated directory
        partial_outputs = {}
        with open(os.path.join(SOURCE_DIR, s + '/feats.scp'), 'r') as f:
            lines = f.readlines()
            for line in lines:
                line = line.split(' ')[0]
                if line in all_inputs:
                    partial_outputs[line] = all_inputs[line]
            assert len(lines) == len(partial_outputs)

        # writiing output with kaldi_io
        ark_scp_output = 'ark:| copy-feats --compress=true ark:- ark,scp:{}/raw_mel_{}.ark,{}/{}/feats.scp'.format(OUTPUT_PATH, str(s), OUTPUT_PATH, str(s))
        with kaldi_io.open_or_fd(ark_scp_output, 'wb') as f:
            for key, mat in tqdm(partial_outputs.items()): 
                kaldi_io.write_mat(f, mat, key=key)

    print('[TIMIT-to-ARK] - All done, saved at \'' + str(OUTPUT_PATH) + '\' exit.')

if __name__ == '__main__':
    main()
    



from .expert import UpstreamExpert
from s3prl.util.download import urls_to_filepaths


def ast(segment_secs: float = 10.24, refresh: bool = False, **kwds):
    ckpt = urls_to_filepaths(
        "https://www.dropbox.com/s/ca0b1v2nlxzyeb4/audioset_10_10_0.4593.pth?dl=1",
        refresh=refresh,
    )
    return UpstreamExpert(ckpt, segment_secs)



import os
import glob
import argparse
from tqdm import tqdm
from pathlib import Path
from collections import defaultdict
from joblib import Parallel, delayed

parser = argparse.ArgumentParser()
parser.add_argument('--libri', help='The root directory of Librispeech')
parser.add_argument('--lexicon', help='The lexicon given by LibriSpeech official website')
parser.add_argument('--output_words', help='The words not in official lexicon')
parser.add_argument('--njobs', type=int, default=12)
args = parser.parse_args()

words_in_lexicon = defaultdict(lambda: False)
with open(args.lexicon, 'r') as file:
    lines = [line[:-1].replace('\t', ' ') for line in file.readlines()]
    for line in lines:
        words_in_lexicon[line.split()[0]] = True

assert os.path.isdir(args.libri)
all_flac = Path(args.libri).rglob('*.flac')

def locate_txt(flac):
    filename = os.path.basename(flac)
    tags = filename.split('.')[0].split('-')
    txt_path = os.path.join(os.path.dirname(flac), f'{tags[0]}-{tags[1]}.trans.txt')
    return txt_path    

all_txt = Parallel(n_jobs=args.njobs)(delayed(locate_txt)(flac) for flac in tqdm(list(all_flac), desc='locate txt'))
all_txt = set(all_txt)

words_not_in_lexicon = []
for txt_path in tqdm(all_txt, desc='find words'):
    with open(txt_path, 'r') as file:
        lines = [line[:-1].replace('\t', ' ') for line in file.readlines()]
        for line in lines:
            idx, transcription = line.split(' ', 1)
            for word in transcription.split():
                if not words_in_lexicon[word]:
                    words_not_in_lexicon.append(f'{word}\n')

words_not_in_lexicon = set(words_not_in_lexicon)
print(f'{len(words_not_in_lexicon)} words not found in LibriSpeech lexicon.')

with open(args.output_words, 'w') as file:
    file.writelines(words_not_in_lexicon)



import json
import pickle
import logging
import pandas as pd
from copy import deepcopy
from dataclasses import dataclass
from collections import defaultdict, OrderedDict

import torch
import torchaudio
from pathlib import Path
from omegaconf import MISSING

from s3prl.task.event_prediction import EventPredictionTask
from s3prl.dataset.hear_timestamp import HearTimestampDatapipe
from s3prl.dataio.sampler import GroupSameItemSampler, FixedBatchSizeBatchSampler

from .hear_fsd import HearFSD

logger = logging.getLogger(__name__)


def dcase_2016_task2(
    target_dir: str,
    cache_dir: str,
    dataset_root: str,
    get_path_only: bool = False,
):
    target_dir: Path = Path(target_dir)
    train_csv = target_dir / "train.csv"
    valid_csv = target_dir / "valid.csv"
    test_csv = target_dir / "test.csv"

    if get_path_only:
        return train_csv, valid_csv, [test_csv]

    dataset_root = Path(dataset_root)
    wav_root = dataset_root / "16000"

    def json_to_csv(json_path: str, csv_path: str, split: str):
        with open(json_path) as fp:
            metadata = json.load(fp)

        data = defaultdict(list)
        for utt in metadata:
            wav_path: Path = (wav_root / split / utt).resolve()
            assert wav_path.is_file()
            info = torchaudio.info(wav_path)
            baseinfo = {
                "record_id": utt,
                "wav_path": str(wav_path),
                "duration": info.num_frames / info.sample_rate,
            }
            for segment in metadata[utt]:
                fullinfo = deepcopy(baseinfo)
                fullinfo[
                    "utt_id"
                ] = f"{baseinfo['record_id']}-{int(segment['start'])}-{int(segment['end'])}"
                fullinfo["labels"] = segment["label"]
                fullinfo["start_sec"] = segment["start"] / 1000
                fullinfo["end_sec"] = segment["end"] / 1000

                for key, value in fullinfo.items():
                    data[key].append(value)

        pd.DataFrame(data=data).to_csv(csv_path, index=False)

    json_to_csv(dataset_root / "train.json", train_csv, "train")
    json_to_csv(dataset_root / "valid.json", valid_csv, "valid")
    json_to_csv(dataset_root / "test.json", test_csv, "test")

    return train_csv, valid_csv, [test_csv]


class HearDcase2016Task2(HearFSD):
    def default_config(self) -> dict:
        return dict(
            start=0,
            stop=None,
            target_dir=MISSING,
            cache_dir=None,
            remove_all_cache=False,
            prepare_data=dict(
                dataset_root=MISSING,
            ),
            build_batch_sampler=dict(
                train=dict(
                    batch_size=5,
                    shuffle=True,
                ),
                valid=dict(
                    item="record_id",
                ),
                test=dict(
                    item="record_id",
                ),
            ),
            build_upstream=dict(
                name="fbank",
            ),
            build_featurizer=dict(
                layer_selections=None,
                normalize=False,
            ),
            build_downstream=dict(
                hidden_layers=2,
            ),
            build_model=dict(
                upstream_trainable=False,
            ),
            build_task=dict(
                prediction_type="multilabel",
                scores=["event_onset_200ms_fms", "segment_1s_er"],
                postprocessing_grid={
                    "median_filter_ms": [250],
                    "min_duration": [125, 250],
                },
            ),
            build_optimizer=dict(
                name="Adam",
                conf=dict(
                    lr=1.0e-4,
                ),
            ),
            build_scheduler=dict(
                name="ExponentialLR",
                gamma=0.9,
            ),
            save_model=dict(),
            save_task=dict(),
            train=dict(
                total_steps=15000,
                log_step=100,
                eval_step=500,
                save_step=500,
                gradient_clipping=1.0,
                gradient_accumulate=1,
                valid_metric="event_onset_200ms_fms",
                valid_higher_better=True,
                auto_resume=True,
                resume_ckpt_dir=None,
            ),
            evaluate=dict(),
        )

    def prepare_data(
        self,
        prepare_data: dict,
        target_dir: str,
        cache_dir: str,
        get_path_only: bool = False,
    ):
        return dcase_2016_task2(
            **self._get_current_arguments(flatten_dict="prepare_data")
        )

    def build_dataset(
        self,
        build_dataset: dict,
        target_dir: str,
        cache_dir: str,
        mode: str,
        data_csv: str,
        encoder_path: str,
        frame_shift: int,
    ):
        with open(encoder_path, "rb") as f:
            encoder = pickle.load(f)

        data = OrderedDict()
        for rowid, row in pd.read_csv(data_csv).iterrows():
            if row["record_id"] not in data:
                data[row["record_id"]] = dict(
                    wav_path=str(row["wav_path"]),
                    start_sec=0.0,
                    end_sec=row["duration"],
                    segments=defaultdict(list),
                )
            data[row["record_id"]]["segments"][str(row["labels"])].append(
                (
                    row["start_sec"],
                    row["end_sec"],
                )
            )
        dataset = HearTimestampDatapipe(feat_frame_shift=frame_shift)(
            data, tools={"category": encoder}
        )
        dataset.set_info({"record_id": "unchunked_id"})
        return dataset

    def build_batch_sampler(
        self,
        build_batch_sampler: dict,
        target_dir: str,
        cache_dir: str,
        mode: str,
        data_csv: str,
        dataset,
    ):
        @dataclass
        class Config:
            train: dict = None
            valid: dict = None
            test: dict = None

        conf = Config(**build_batch_sampler)
        if mode == "train":
            return FixedBatchSizeBatchSampler(dataset, **(conf.train or {}))
        elif mode == "valid":
            return GroupSameItemSampler(dataset, **(conf.valid or {}))
        elif mode == "test":
            return GroupSameItemSampler(dataset, **(conf.test or {}))
        else:
            raise ValueError(f"Unsupported mode: {mode}")

    def build_task(
        self,
        build_task: dict,
        model: torch.nn.Module,
        encoder,
        valid_df: pd.DataFrame = None,
        test_df: pd.DataFrame = None,
    ):
        def df_to_events(df: pd.DataFrame):
            data = {}
            for rowid, row in df.iterrows():
                record_id = row["record_id"]
                if not record_id in data:
                    data[record_id] = []
                data[record_id].append(
                    {
                        "start": row["start_sec"] * 1000,
                        "end": row["end_sec"] * 1000,
                        "label": row["labels"],
                    }
                )
            return data

        valid_events = None if valid_df is None else df_to_events(valid_df)
        test_events = None if test_df is None else df_to_events(test_df)

        return EventPredictionTask(
            model,
            encoder,
            valid_target_events=valid_events,
            test_target_events=test_events,
            **build_task,
        )



# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ preprocess_mosei.py ]
#   Synopsis     [ preprocessing for the MOSEI dataset ]
#   Author       [ Andy T. Liu (Andi611) ]
#   Copyright    [ Copyleft(c), Speech Lab, NTU, Taiwan ]
"""*********************************************************************************************"""


###############
# IMPORTATION #
###############
import os
import shutil
import glob
import sys
import pickle
import argparse
import numpy as np
import pandas as pd
from tqdm import tqdm
from pathlib import Path
import shutil
from joblib import Parallel, delayed
from utility.asr import encode_target
from utility.audio import extract_feature, mel_dim, num_freq
from pydub import AudioSegment


def bracket_underscore(string):
    split = string.split('[')
    utterance_name = split[0]
    number = int(split[1].split(']')[0])
    string = utterance_name + '_' + str(number + 1)
    return string

def underscore_bracket(string):
    split = string.split('_')
    number = int(split[-1][:-4])
    utterance_name = '_'.join(split[:-1])
    string = utterance_name + '[' + str(number - 1) + ']'
    return string


def get_preprocess_args():
    parser = argparse.ArgumentParser(description='preprocess arguments for LibriSpeech dataset.')
    parser.add_argument('--data_path', default='/home/leo/d/datasets/MOSEI/Raw/Audio/Full/WAV_16000', type=str, help='Path to MOSEI non-segmented WAV files')
    parser.add_argument('--output_path', default='../../data/mosei', type=str, help='Path to store segmented flac and npys. Should already contains mosei_no_semi.csv', required=False)
    args = parser.parse_args()
    return args


def segment_mosei(args):
    output_dir = args.output_path
    mosei_summary = os.path.join(output_dir, 'mosei_no_semi.csv')
    flac_dir = os.path.join(output_dir, 'flac')
    assert os.path.exists(mosei_summary), 'Output path should already be created with a mosei_no_semi.csv inside it'
    for target_dir in [flac_dir]:
        if os.path.exists(target_dir):
            decision = input(f'{target_dir} already exists. Remove it? [Y/N]: ')
            if decision.upper() == 'Y':
                shutil.rmtree(target_dir)
                print(f'{target_dir} removed')
            else:
                print('Abort')
                exit(0)
        os.makedirs(target_dir)

    df = pd.read_csv(mosei_summary)

    for index, row in df.iterrows():
        underscore = row.key
        wavname = f'{row.filename}.wav'
        wavpath = os.path.join(args.data_path, wavname)
        assert os.path.exists(wavpath), f'wav not exists: {wavpath}'
        wav = AudioSegment.from_wav(wavpath)

        start = int(row.start * 1000)
        end = int(row.end * 1000)
        assert start >= 0, f'{underscore} has negative start time'
        assert end >= 0, f'{underscore} has negative end time'
        seg_wav = wav[start:end]
        seg_flacpath = os.path.join(flac_dir, f'{underscore}.flac')
        seg_wav.export(seg_flacpath, format='flac', parameters=['-ac', '1', '-sample_fmt', 's16', '-ar', '16000'])


########
# MAIN #
########
def main():
    # get arguments
    args = get_preprocess_args()

    # Acoustic Feature Extraction
    segment_mosei(args)


if __name__ == '__main__':
    main()




from random import shuffle 
import os
from glob import glob
import shutil
import re
import tqdm
from multiprocessing import Pool

from normalise import normalise

months = {'jan.': 'January', 'feb.': 'February', 'mar.': 'March', 'apr.': 'April', 'may': 'May', 'jun.': 'June', 'jul.': 'July', 'aug.': 'August', 'sep.': 'September', 'oct.': 'October', 'nov.': 'November', 'dec.': 'December', 'jan': 'January', 'feb': 'February', 'mar': 'March', 'apr': 'April', 'jun': 'June', 'jul': 'July', 'aug': 'August', 'sep': 'September', 'oct': 'October', 'nov': 'November', 'dec': 'December'}

replace_words = {'&': 'and', '¡':'', 'r&b':'R and B', 'funtime':'fun time', 'español':'espanol', "'s":'s', 'palylist':'playlist'}
replace_vocab = {'ú':'u', 'ñ':'n', 'Ō':'O', 'â':'a'}

reservations = {'chyi':'chyi', 'Pre-Party':'pre party', 'Chu':'Chu', 'B&B':'B and B', '0944':'nine four four', 'Box':'Box', 'ain’t':'am not', 'Zon':'Zon', 'Yui':'Yui', 'neto':'neto', 'skepta':'skepta', '¡Fiesta':'Fiesta', 'Vue':'Vue', 'iheart':'iheart', 'disco':'disco'}
same = "klose la mejor música para tus fiestas dubstep dangles drejer listas".split(' ')
for word in same:
    reservations[word] = word

def word_normalise(words):
    ret = []
    for word in words:
        if word.lower() in months:
            word = months[word.lower()]
        if word.lower() in replace_words:
            word = replace_words[word.lower()]
        for regex in replace_vocab:
            word = re.sub(regex, '', word)
        #word = re.sub(r'(\S)([\.\,\!\?])', r'\1 \2', word)
        word = re.sub(r'[\.\,\!\?;\/]', '', word)
        ret.append(word)
    return ret

def sent_normalise(text, slots_split=None):
    norm_slots, norm_texts = [], []
    text_split = text.split(' ')
    if slots_split is None:
        slots_split = ['O']*len(text_split)
    for idx in range(len(text_split)):
        if text_split[idx] in '.,!?;/]':
            continue
        if text_split[idx] in reservations:
            for word in reservations[text_split[idx]].split(' '):
                norm_texts.append(word)
                norm_slots.append(slots_split[idx])
            continue
        norm_text = normalise(word_normalise([text_split[idx]]), variety="AmE", verbose=False)
        for phrase in norm_text:
            if phrase == '':
                continue
            for word in re.split(r' |\-', phrase):
                word = re.sub(r'[\.\,\!\?;\/]', '', word)
                if word == '':
                    continue
                norm_texts.append(word)
                norm_slots.append(slots_split[idx])
    return norm_slots, norm_texts


def process_raw_snips_file(file, out_f):
    with open(file) as f:
        content = f.readlines()
    content = [x.strip() for x in content]

    with open(out_f, 'w') as f:
        for cnt, line in enumerate(content):
            text = line.split(' <=> ')[0]
            intent = line.split(' <=> ')[1]
            #[r.split(':')[0] if len(r.split(':')) == 2 else ' ' for r in x.split()]
            text_split = [x.replace('::', ':').split(':')[0] if len(x.replace('::', ':').split(':')) == 2 else ' ' for x in text.split()]
            text_entities = ' '.join(text_split)
            slots_split = [x.replace('::', ':').split(':')[1] for x in text.split()]
            slots_entities = ' '.join(slots_split)
            assert len(text_split) == len(slots_split), (text_split, slots_split)
            f.write('%d | BOS %s EOS | O %s | %s\n' % (cnt, text_entities, slots_entities, intent))

def remove_IBO_from_snipt_vocab_slot(in_f, out_f):
    with open(in_f) as f:
        content = f.readlines()
    content = [x.strip() for x in content]
    # get rid of BIO tag from the slots 
    for idx, line in enumerate(content):
        if line != 'O':
            content[idx] = line[len('B-'):]
    content = set(content) # remove repeating slots 

    with open(out_f, 'w') as f:
        for line in content:
            f.write('%s\n' % line)

def process_daniel_snips_file(content):
    content = [x.strip() for x in content]
    utt_ids = [x.split('\t', 1)[0] for x in content]
   
    valid_uttids = [x for x in utt_ids if x.split('-')[1] == 'valid']
    test_uttids  = [x for x in utt_ids if x.split('-')[1] == 'test']
    train_uttids = [x for x in utt_ids if x.split('-')[1] == 'train']

    utt2text, utt2slots, utt2intent = {}, {}, {}
    assert len(utt_ids) == len(set(utt_ids))

    # create utt2text, utt2slots, utt2intent
    for line in content:
        uttid, text, slots, intent = line.split('\t')
        if len(text.split()) != len(slots.split()): # detect 'empty' in text 
            assert len(text.split('  ')) == 2
            empty_idx = text.split().index(text.split('  ')[0].split()[-1]) + 1 
            slots_list = slots.split()
            del slots_list[empty_idx]
            cleaned_slots = ' '.join(slots_list)
            assert len(text.split()) == len(slots_list)
            cleaned_text = ' '.join(text.split())
            #print(cleaned_text, cleaned_slots)
        else: 
            (cleaned_text, cleaned_slots) = (text, slots)

        # get rid of the 'intent/' from all slot values 
        cleaned_slots = ' '.join([x.split('/')[1] if x != 'O' else x for x in cleaned_slots.split()])
        # strip the whitespaces before punctuations 
        #cleaned_text = re.sub(r'\s([?.!,"](?:\s|$))', r'\1', cleaned_text)
        
        utt2text[uttid]   = cleaned_text
        utt2slots[uttid]  = cleaned_slots
        utt2intent[uttid] = intent

    test_utt2text, test_utt2slots, test_utt2intent    = {}, {}, {}
    valid_utt2text, valid_utt2slots, valid_utt2intent = {}, {}, {}
    train_utt2text, train_utt2slots, train_utt2intent = {}, {}, {}
    for utt in valid_uttids:
         valid_utt2text[utt] = utt2text[utt]
         valid_utt2slots[utt] = utt2slots[utt]
         valid_utt2intent[utt] = utt2intent[utt]
    for utt in test_uttids:
         test_utt2text[utt] = utt2text[utt]
         test_utt2slots[utt] = utt2slots[utt]
         test_utt2intent[utt] = utt2intent[utt] 
    for utt in train_uttids:
         train_utt2text[utt] = utt2text[utt]
         train_utt2slots[utt] = utt2slots[utt]
         train_utt2intent[utt] = utt2intent[utt]
   
    assert len(set(valid_utt2intent.values())) == len(set(test_utt2intent.values())) == len(set(train_utt2intent.values())) == 7
    assert len(valid_utt2intent.keys()) == len(test_utt2intent.keys()) == 700
    assert len(train_utt2intent.keys()) == 13084
  
    def __return_set_of_slots(utt2slots):
        all_slots = []
        for slot in utt2slots.values():
            all_slots.extend(slot.split())
        unique_slots = set(all_slots)

        return unique_slots
    
    assert len(__return_set_of_slots(valid_utt2slots)) == len(__return_set_of_slots(test_utt2slots)) == \
        len(__return_set_of_slots(train_utt2slots)) == 40
  
    return (train_utt2text, train_utt2slots, train_utt2intent), \
           (valid_utt2text, valid_utt2slots, valid_utt2intent), \
           (test_utt2text, test_utt2slots, test_utt2intent)

def map_and_link_snips_audio(snips_audio_dir, link_dir):

    # traverse through snips_audio_dir
    result = [y for x in os.walk(snips_audio_dir) for y in glob(os.path.join(x[0], '*.mp3'))]

    for path in result: 
        person   = path.split('/')[8].split('_')[1]
        filename = path.split('/')[-1]
        if filename[:5] != 'snips':
            continue
        uttid = filename.split('.')[0]
        new_uttid = person + '-' + filename
        partition = uttid.split('-')[1]
        destination = os.path.join(link_dir, partition, new_uttid)
        shutil.copyfile(path, destination)

def create_multispk_for_snips(output_dir):
    speakers = "Aditi Amy Brian Emma Geraint Ivy Joanna Joey Justin Kendra Kimberly Matthew Nicole Raveena Russell Salli".split(' ')
    dataset_info = [{'split':'test', 'num_utts':700}, {'split':'valid', 'num_utts':700}, {'split':'train', 'num_utts':13084}]
    test_out_f = open(os.path.join(output_dir, 'all.iob.snips.txt'), 'w')
    for data in dataset_info:
        num_utts = data['num_utts']
        split = data['split']
        with open(os.path.join(output_dir, 'single-matched-snips.%s.w-intent'%split)) as f:
            content = f.readlines()
        utt2line = {x.strip().split()[0]:x.strip() for x in content}
        for spk in speakers:
            for num in range(num_utts):
                uttid  = "%s-snips-%s-%d"%(spk, split, num) #mp3.split('/')[-1].split('.')[0]
                line   = utt2line["snips-%s-%d"%(split, num)] #'-'.join(uttid.split('-')[1:])]
                text   = line.split('\t')[1].upper()
                slots  = line.split('\t')[2]
                intent = line.split('\t')[3] 
                test_out_f.write('%s BOS %s EOS\tO %s %s\n' % (uttid, text, slots, intent))
    test_out_f.close()

def apply_text_norm_and_modify_slots(all_tsv, output_dir):
    
    train_dirs, valid_dirs, test_dirs = process_daniel_snips_file(all_tsv)
    # test 
    test_file = open(os.path.join(output_dir, 'single-matched-snips.test.w-intent'), 'w')
    vocab_slot = {}
    for uttid in tqdm.tqdm(test_dirs[0].keys(), desc='Text Normalising on testing set'):
        text = test_dirs[0][uttid]
        slots = test_dirs[1][uttid]
        intent = test_dirs[2][uttid]
        slots_split = slots.split()
        for s in slots_split:
            vocab_slot.setdefault(s, 0)
            vocab_slot[s] += 1

        norm_slots, norm_texts = sent_normalise(text, slots_split)
        assert len(norm_texts) == len(norm_slots), (norm_texts, norm_slots)
     
        # write to file
        test_file.write('%s\t%s\t%s\t%s\n' % (uttid, ' '.join(norm_texts).upper(), ' '.join(norm_slots), intent))
    test_file.close()

    # valid 
    valid_file = open(os.path.join(output_dir, 'single-matched-snips.valid.w-intent'), 'w')
    for uttid in tqdm.tqdm(valid_dirs[0].keys(), desc='Text Normalising on validation set'):
        text = valid_dirs[0][uttid]
        slots = valid_dirs[1][uttid]
        intent = valid_dirs[2][uttid]
        slots_split = slots.split()
        for s in slots_split:
            vocab_slot.setdefault(s, 0)
            vocab_slot[s] += 1

        norm_slots, norm_texts = sent_normalise(text, slots_split)
        assert len(norm_texts) == len(norm_slots), (norm_texts, norm_slots)
     
        # write to file
        valid_file.write('%s\t%s\t%s\t%s\n' % (uttid, ' '.join(norm_texts).upper(), ' '.join(norm_slots), intent))
    valid_file.close()

    # train 
    train_file = open(os.path.join(output_dir, 'single-matched-snips.train.w-intent'), 'w')
    for uttid in tqdm.tqdm(train_dirs[0].keys(), desc='Text Normalising on training set'):
        text = train_dirs[0][uttid]
        slots = train_dirs[1][uttid]
        intent = train_dirs[2][uttid]
        slots_split = slots.split()
        for s in slots_split:
            vocab_slot.setdefault(s, 0)
            vocab_slot[s] += 1

        norm_slots, norm_texts = sent_normalise(text, slots_split)
        assert len(norm_texts) == len(norm_slots), (norm_texts, norm_slots)
     
        # write to file
        train_file.write('%s\t%s\t%s\t%s\n' % (uttid, ' '.join(norm_texts).upper(), ' '.join(norm_slots), intent))
    train_file.close()
    vocab_file = open(os.path.join(output_dir, 'slots.txt'), 'w')
    vocab_file.write('\n'.join(sorted(list(vocab_slot.keys()), key=lambda x:vocab_slot[x], reverse=True)))

def sox_func(inputs):
    files, root, out_root, speaker = inputs
    for name in tqdm.tqdm(files, desc='Process for speaker: '+speaker):
        if name.endswith(".mp3"):
            split = name.split('-')[1]
            out_dir = os.path.join(out_root, split)
            os.makedirs(out_dir, exist_ok=True)
            orig_file = os.path.join(root, name)
            new_file  = os.path.join(out_dir, speaker+'-'+name.split('/')[-1].split('.')[0] + '.wav')
            bashCommand = "sox " + orig_file + " -t wav -c 1 -r 16000 -b 16 -e signed-integer " + new_file
            r = os.popen(bashCommand).read()

def sox_mp3_to_wav(in_root, out_root):
    
    os.makedirs(out_root, exist_ok=True)
    pool = Pool(16)
    inputs = []
    for root, dirs, files in os.walk(in_root):
        print('[Processing] enter directory %s'%root)
        if not len(files):
            continue
        speaker = root.split('/')[-2].split('_')[1]
        print('[Processing] process %d audio files from speaker %s'%(len(files), speaker))
        inputs.append((files, root, out_root, speaker))
    pool.map(sox_func, inputs)

if __name__ == '__main__':

    import sys, os
    mode = sys.argv[1]
    if mode == 'text':
        repo_dir = sys.argv[2]
        dump_dir = sys.argv[3]
        os.makedirs(dump_dir, exist_ok=True)

        content = []
        content += open(os.path.join(repo_dir, 'data/nlu_annotation/valid')).readlines()[1:]
        content += open(os.path.join(repo_dir, 'data/nlu_annotation/test')).readlines()[1:]
        content += open(os.path.join(repo_dir, 'data/nlu_annotation/train')).readlines()[1:]
        apply_text_norm_and_modify_slots(content, dump_dir)
        create_multispk_for_snips(dump_dir)
    elif mode == 'audio':
        audio_dir = sys.argv[2]
        dump_dir = sys.argv[3]
        # Step: sox the snips *.mp3 to the correct format 
        sox_mp3_to_wav(audio_dir, dump_dir)
    else:
        print('Usage: python preprocess.py [text|audio] [data_path] [dump_path]')



from collections import OrderedDict
from s3prl.dataset.common_pipes import LoadAudio
from s3prl.util.pseudo_data import pseudo_audio


def test_load_audio():
    with pseudo_audio([8, 20, 15], 16000) as (audio_files, num_samples):
        data = OrderedDict(
            {i: dict(wav_path=file) for i, file in enumerate(audio_files)}
        )
        dataset = LoadAudio()(data)
        assert len(dataset[0]["wav"]) == num_samples[0]

        data[0]["start_sec"] = 1
        data[0]["end_sec"] = 3
        dataset = LoadAudio()(OrderedDict({0: data[0]}))
        assert len(dataset[0]["wav"]) == 32000



"""
The setting of Superb PR

Authors
  * Heng-Jui Chang 2022
  * Shu-wen Yang 2022
"""

import pickle
from collections import OrderedDict
from dataclasses import dataclass
from pathlib import Path

import pandas as pd
from omegaconf import MISSING

from s3prl.dataset.speech2phoneme_pipe import Speech2PhonemePipe
from s3prl.dataio.encoder.tokenizer import default_phoneme_tokenizer
from s3prl.nn.linear import FrameLevelLinear
from s3prl.dataio.sampler import FixedBatchSizeBatchSampler, SortedSliceSampler

from .superb_asr import SuperbASR

__all__ = [
    "SuperbPR",
]


class SuperbPR(SuperbASR):
    def default_config(self) -> dict:
        return dict(
            start=0,
            stop=None,
            target_dir=MISSING,
            cache_dir=None,
            remove_all_cache=False,
            prepare_data=dict(
                dataset_root=MISSING,
                train_sets=["train-clean-100"],
                valid_sets=["dev-clean"],
                test_sets=["test-clean"],
            ),
            prepare_tokenizer_data=dict(),
            build_tokenizer=dict(),
            build_dataset=dict(),
            build_batch_sampler=dict(
                train=dict(
                    batch_size=16,
                    max_length=300000,
                ),
                valid=dict(
                    batch_size=1,
                ),
                test=dict(
                    batch_size=1,
                ),
            ),
            build_upstream=dict(
                name="fbank",
            ),
            build_featurizer=dict(
                layer_selections=None,
                normalize=False,
            ),
            build_downstream=dict(
                hidden_size=256,
            ),
            build_model=dict(
                upstream_trainable=False,
            ),
            build_task=dict(
                log_metrics=["per"],
            ),
            build_optimizer=dict(
                name="Adam",
                conf=dict(
                    lr=1.0e-2,
                ),
            ),
            build_scheduler=dict(
                name="ExponentialLR",
                gamma=0.9,
            ),
            save_model=dict(
                extra_conf=dict(
                    build_downstream_conf="${build_downstream}"
                ),  # This is redundant for ASR. Just to show how to clone other fields
            ),
            save_task=dict(),
            train=dict(
                total_steps=200000,
                log_step=100,
                eval_step=2000,
                save_step=500,
                gradient_clipping=1.0,
                gradient_accumulate=1,
                valid_metric="per",
                valid_higher_better=False,
                auto_resume=True,
                resume_ckpt_dir=None,
            ),
            evaluate=dict(),
        )

    def build_tokenizer(
        self,
        build_tokenizer: dict,
        target_dir: str,
        cache_dir: str,
        tokenizer_data_path: str,
        get_path_only: bool = False,
    ):
        """
        Build the tokenizer from the data prepared by :obj:`prepare_tokenizer_data`
        By default use the :obj:`default_phoneme_tokenizer`

        Args:
            build_tokenizer (dict): same in :obj:`default_config`, not used
            target_dir (str): Current experinment directory
            cache_dir (str): If the parsing or preprocessing takes too long time, you can save
                the temporary files into this directory. This directory is expected to be shared
                across different training sessions (different hypers and :code:`target_dir`)
            tokenizer_data_path (str): The text file from :obj:`prepare_tokenizer_data`
            get_path_only (str): Directly return the filepaths no matter they exist or not.

        Returns:
            str

            filepath of the pickled :obj:`s3prl.dataio.encoder.tokenizer.Tokenizer`
        """

        tokenizer_path = Path(target_dir) / "default_phone_tokenizer.pkl"
        with tokenizer_path.open("wb") as f:
            pickle.dump(default_phoneme_tokenizer(), f)
        return tokenizer_path

    def build_dataset(
        self,
        build_dataset: dict,
        target_dir: str,
        cache_dir: str,
        mode: str,
        data_csv: str,
        tokenizer_path: str,
    ):
        data_points = OrderedDict()
        csv = pd.read_csv(data_csv)
        for _, row in csv.iterrows():
            data_points[row["id"]] = {
                "wav_path": row["wav_path"],
                "transcription": row["transcription"],
            }

        with open(tokenizer_path, "rb") as f:
            tokenizer = pickle.load(f)

        dataset = Speech2PhonemePipe()(
            data_points,
            tools={"tokenizer": tokenizer},
        )
        return dataset

    def build_batch_sampler(
        self,
        build_batch_sampler: dict,
        target_dir: str,
        cache_dir: str,
        mode: str,
        data_csv: str,
        dataset,
    ):
        """
        Return the batch sampler for torch DataLoader.

        Args:
            build_batch_sampler (dict): same in :obj:`default_config`

                ====================  ====================
                key                   description
                ====================  ====================
                train                 (dict) - arguments for :obj:`SortedSliceSampler`
                valid                 (dict) - arguments for :obj:`FixedBatchSizeBatchSampler`
                test                  (dict) - arguments for :obj:`FixedBatchSizeBatchSampler`
                ====================  ====================

            target_dir (str): Current experiment directory
            cache_dir (str): If the preprocessing takes too long time, save
                the temporary files into this directory. This directory is expected to be shared
                across different training sessions (different hypers and :code:`target_dir`)
            mode (str): train/valid/test
            data_csv (str): the :code:`mode` specific csv from :obj:`prepare_data`
            dataset: the dataset from :obj:`build_dataset`

        Returns:
            batch sampler for torch DataLoader
        """

        @dataclass
        class Config:
            train: dict = None
            valid: dict = None
            test: dict = None

        conf = Config(**build_batch_sampler)

        if mode == "train":
            sampler = SortedSliceSampler(dataset, **(conf.train or {}))
        elif mode == "valid":
            sampler = FixedBatchSizeBatchSampler(dataset, **(conf.valid or {}))
        elif mode == "test":
            sampler = FixedBatchSizeBatchSampler(dataset, **(conf.test or {}))

        return sampler

    def build_downstream(
        self,
        build_downstream: dict,
        downstream_input_size: int,
        downstream_output_size: int,
        downstream_input_stride: int,
    ):
        """
        Return the task-specific downstream model.
        By default build the :obj:`FrameLevelLinear`

        Args:
            build_downstream (dict): same in :obj:`default_config`,
                supports arguments in :obj:`FrameLevelLinear`
            downstream_input_size (int): the required input size of the model
            downstream_output_size (int): the required output size of the model
            downstream_input_stride (int): the input feature's stride (from 16 KHz)

        Returns:
            :obj:`s3prl.nn.interface.AbsFrameModel`
        """
        return FrameLevelLinear(
            downstream_input_size, downstream_output_size, **build_downstream
        )



"""
Common model and loss in pure torch.nn.Module with torch dependency only
"""

from .upstream import S3PRLUpstream, Featurizer
from .common import FrameLevel, UtteranceLevel
from .linear import FrameLevelLinear, MeanPoolingLinear
from .pooling import MeanPooling, TemporalAveragePooling, TemporalStatisticsPooling
from .rnn import RNNEncoder, SuperbDiarizationModel
from .speaker_loss import amsoftmax, softmax
from .speaker_model import SuperbXvector, XVectorBackbone
from .beam_decoder import BeamDecoder


__all__ = [
    "S3PRLUpstream",
    "Featurizer",
    "FrameLevel",
    "UtteranceLevel",
    "FrameLevelLinear",
    "MeanPoolingLinear",
    "MeanPooling",
    "TemporalAveragePooling",
    "TemporalStatisticsPooling",
    "RNNEncoder",
    "SuperbDiarizationModel",
    "amsoftmax",
    "softmax",
    "SuperbXvector",
    "XVectorBackbone",
    "BeamDecoder",
]



from pathlib import Path

# HACK: SummaryWriter must be imported at the begining or else it will lead to core dumped
# This is a known issue: https://github.com/pytorch/pytorch/issues/30651
from torch.utils.tensorboard.writer import SummaryWriter

from . import problem, hub, dataset, metric, nn, task, util, dataio

with (Path(__file__).parent.resolve() / "version.txt").open() as file:
    __version__ = file.read().strip()



# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ ark2voxceleb.py ]
#   Synopsis     [ process the .ark file preprocessed by kaldi for our dataloader ]
#   Author       [ Andy T. Liu (Andi611) ]
#   Copyright    [ Copyleft(c), Speech Lab, NTU, Taiwan ]
#   Reference    [ https://github.com/nttcslab-sp/kaldiio ]
"""*********************************************************************************************"""

###############
# IMPORTATION #
###############
import os
import pickle
import kaldi_io
import operator
import numpy as np
import pandas as pd
from tqdm import tqdm


############
# SETTINGS #
############
KALDI_PATH = os.path.join('../kaldi/egs/voxceleb/v1/data/') # this needs to be generated by the kaldi scripts
OUTPUT_DIR = '../data/voxceleb_mfcc_kaldi' 


############
# CONSTANT #
############
SETS = ['train'] # can be any subset of: ['train', 'dev', 'test']


########
# MAIN #
########
def main():
    if not os.path.isdir(KALDI_PATH):
        print('CHANGE THIS TO YOUR OWN KALDI PATH: ', KALDI_PATH)
        print('Please run the kaldi scripts first to generate kaldi data directory.')
        exit()

    if not os.path.isdir(OUTPUT_DIR):
        os.mkdir(OUTPUT_DIR)

    # read data from the preprocessed kaldi directory
    for s in SETS:
        print('Preprocessing', s, 'data...')
        output = {}
        cur_dir = os.path.join(OUTPUT_DIR, s)
        if not os.path.isdir(cur_dir): os.mkdir(cur_dir)

        path = os.path.join(KALDI_PATH, s + '/feats.scp') # kaldi data is already sorted
        for key, mat in tqdm(kaldi_io.read_mat_scp(path)): # (key, mat) is returned

            array = np.asarray(mat).astype('float32')
            np.save(os.path.join(cur_dir, key), array)
            output[os.path.join(s, key + '.npy')] = len(array)

        output = sorted(output.items(), key=operator.itemgetter(1), reverse=True)
        df = pd.DataFrame(data={'file_path':[fp for fp, l in output], 'length':[l for fp, l in output], 'label':'None'})
        df.to_csv(os.path.join(OUTPUT_DIR, s + '.csv'))

    print('[ARK-TO-VOXCELEB] - All done, saved at \'' + str(OUTPUT_DIR) + '\', exit.')
    exit()

if __name__ == '__main__':
    main()


import torch
from collections import OrderedDict

from s3prl.dataset.base import AugmentedDynamicItemDataset
from s3prl.dataset.chunking import UnfoldChunkByFrame, UnfoldChunkBySec
from s3prl.dataset.common_pipes import LoadAudio
from s3prl.dataset.multiclass_tagging import BuildMultiClassTagging
from s3prl.util.pseudo_data import pseudo_audio


def test_chunk_pipe():
    with pseudo_audio([8, 20, 15], 16000) as (audio_files, num_samples):
        data = {
            i: dict(wav_path=file, start_sec=0, end_sec=num_sample / 16000)
            for i, (file, num_sample) in enumerate(zip(audio_files, num_samples))
        }
        data = OrderedDict(data)
        dataset = UnfoldChunkByFrame(
            min_chunk_frames=25, step_frames=25, feat_frame_shift=320
        )(data)
        assert dataset[0]["end_sec"] - dataset[0]["start_sec"] == 25 * 320 / 16000
        dataset = LoadAudio()(dataset)
        assert len(dataset[0]["wav"]) == 25 * 320

    with pseudo_audio([8, 20, 15], 16000) as (audio_files, num_samples):
        data = {
            i: dict(wav_path=file, start_sec=0, end_sec=num_sample / 16000)
            for i, (file, num_sample) in enumerate(zip(audio_files, num_samples))
        }
        data = OrderedDict(data)
        dataset = UnfoldChunkBySec(min_chunk_secs=2.5, step_secs=2.5)(data)
        assert (
            dataset.data[list(dataset.data.keys())[0]]["end_sec"]
            - dataset.data[list(dataset.data.keys())[0]]["start_sec"]
            == 2.5
        )
        dataset = LoadAudio()(dataset)
        assert len(dataset[0]["wav"]) == 2.5 * 16000


def test_multiclass_pipe():
    with pseudo_audio([8, 20, 15], 16000) as (audio_files, num_samples):
        data = {
            "another": dict(
                wav_path=audio_files[0],
                start_sec=0,
                end_sec=5.3,
                segments={
                    "spk1": [(0, 3.2), (3.6, 4.3)],
                    "spk2": [(1.1, 4.1)],
                },
            ),
            "best": dict(
                wav_path=audio_files[1],
                start_sec=0,
                end_sec=20,
                segments={
                    "spk1": [(0, 1), (2, 3)],
                    "spk3": [(0.5, 1.1), (1.5, 2.6), (3.5, 10)],
                },
            ),
        }
        data = OrderedDict(data)
        dataset = BuildMultiClassTagging(feat_frame_shift=160)(data)
        assert len(dataset) == 2
        assert dataset[0]["multiclass_tag"].shape == (round(5.3 * 16000 / 160), 2)

        dataset = BuildMultiClassTagging(feat_frame_shift=320)(data)
        assert len(dataset) == 2
        assert dataset[0]["multiclass_tag"].shape == (round(5.3 * 16000 / 320), 2)

        dataset = UnfoldChunkBySec(min_chunk_secs=4, step_secs=4)(data)
        dataset: AugmentedDynamicItemDataset = BuildMultiClassTagging(
            feat_frame_shift=160
        )(dataset)
        assert dataset[0]["multiclass_tag"].shape == (round(4 * 16000 / 160), 2)
        assert dataset[1]["multiclass_tag"].shape == (round(1.3 * 16000 / 160), 2)
        encoder = dataset.get_tool("all_tag_category")
        first_spk = (
            torch.from_numpy(dataset[0]["multiclass_tag"]).argmax(dim=0)[0].item()
        )
        encoder.decode(first_spk) == "spk1"



"""
Limit the maximum timestamps in a batch to realize dynamic batching.

Authors:
  * Shu-wen Yang 2022
"""

import torch
from joblib import Parallel, delayed
from tqdm import tqdm

__all__ = [
    "MaxTimestampBatchSampler",
]


class MaxTimestampBatchSampler:
    """
    The reduced timestamps for a batch should not exceed the max_timestamp.
    If shuffled, each indices are first shuffled before aggregated into batches
    """

    def __init__(
        self,
        dataset,
        max_timestamp: int,
        shuffle: bool = False,
        seed: int = 12345678,
        get_length_func: callable = None,
        reduce_func: callable = None,
    ) -> None:
        get_length_func = get_length_func or self.get_length
        timestamps = get_length_func(dataset)

        self.timestamps = timestamps
        self.max_timestamp = max_timestamp
        self.shuffle = shuffle
        self.seed = seed
        self.epoch = 0
        self.reduce_func = reduce_func or self._default_reduce_func

    @staticmethod
    def get_length(dataset):
        import torchaudio

        torchaudio.set_audio_backend("sox_io")

        lengths = []
        with dataset.output_keys_as(["wav_path"]):
            for data_index, item in enumerate(
                tqdm(dataset, desc="Read wav_path audio length")
            ):
                info = torchaudio.info(item["wav_path"])
                lengths.append(info.num_frames)
        return lengths

    @staticmethod
    def _default_reduce_func(timestamps):
        return max(timestamps) * len(timestamps)

    def set_epoch(self, epoch: int):
        self.epoch = epoch

    def _evaluate_reduced_timestamps(self, batch_indices):
        return self.reduce_func([self.timestamps[indice] for indice in batch_indices])

    def __iter__(self):
        if self.shuffle:
            generator = torch.Generator()
            generator.manual_seed(self.epoch + self.seed)
            indices = torch.randperm(len(self.timestamps), generator=generator).tolist()
        else:
            indices = list(range(len(self.timestamps)))

        batch = []
        for indice in indices:
            try_new_batch = batch + [indice]
            if self._evaluate_reduced_timestamps(try_new_batch) <= self.max_timestamp:
                batch = try_new_batch
            elif len(batch) == 0:
                raise ValueError(
                    f"There is a single timestamp {self.timestamps[indice]} larger than "
                    f"max_timestamp {self.max_timestamp}. Please increase "
                    "the max_timestamp."
                )
            else:
                yield batch
                batch = [indice]

        if len(batch) > 0:
            yield batch

    def __len__(self):
        return len(list(iter(self)))



"""
The setting of Superb SF

Authors
  * Yung-Sung Chuang 2021
  * Heng-Jui Chang 2022
  * Shu-wen Yang 2022
"""

from dataclasses import dataclass
from pathlib import Path
from typing import List

import pandas as pd
from omegaconf import MISSING

from s3prl.dataio.corpus.snips import SNIPS
from s3prl.dataio.sampler import FixedBatchSizeBatchSampler
from s3prl.util.download import _urls_to_filepaths

from .superb_asr import SuperbASR

VOCAB_URL = "https://huggingface.co/datasets/s3prl/SNIPS/raw/main/character.txt"
SLOTS_URL = "https://huggingface.co/datasets/s3prl/SNIPS/raw/main/slots.txt"

__all__ = [
    "audio_snips_for_slot_filling",
    "SuperbSF",
]


def audio_snips_for_slot_filling(
    target_dir: str,
    cache_dir: str,
    dataset_root: str,
    train_speakers: List[str],
    valid_speakers: List[str],
    test_speakers: List[str],
    get_path_only: bool = False,
):
    target_dir = Path(target_dir)

    train_path = target_dir / f"train.csv"
    valid_path = target_dir / f"valid.csv"
    test_paths = [target_dir / f"test.csv"]

    if get_path_only:
        return train_path, valid_path, test_paths

    corpus = SNIPS(dataset_root, train_speakers, valid_speakers, test_speakers)
    train_data, valid_data, test_data = corpus.data_split

    def dict_to_csv(data_dict, csv_path):
        keys = sorted(list(data_dict.keys()))
        fields = sorted(data_dict[keys[0]].keys())
        data = dict()
        for field in fields:
            data[field] = []
            for key in keys:
                data[field].append(data_dict[key][field])
        data["id"] = keys
        df = pd.DataFrame(data)
        df.to_csv(csv_path, index=False)

    dict_to_csv(train_data, train_path)
    dict_to_csv(valid_data, valid_path)
    dict_to_csv(test_data, test_paths[0])

    return train_path, valid_path, test_paths


class SuperbSF(SuperbASR):
    def default_config(self) -> dict:
        return dict(
            start=0,
            stop=None,
            target_dir=MISSING,
            cache_dir=None,
            remove_all_cache=False,
            prepare_data=dict(
                dataset_root=MISSING,
                train_speakers=[
                    "Ivy",
                    "Joanna",
                    "Joey",
                    "Justin",
                    "Kendra",
                    "Kimberly",
                    "Matthew",
                    "Salli",
                ],
                valid_speakers=["Aditi", "Amy", "Geraint", "Nicole"],
                test_speakers=["Brian", "Emma", "Raveena", "Russell"],
            ),
            prepare_tokenizer_data=dict(),
            build_tokenizer=dict(
                tokenizer_name=None,
                vocab_type="character",
                vocab_file=VOCAB_URL,
                slots_file=SLOTS_URL,
            ),
            build_dataset=dict(),
            build_batch_sampler=dict(
                train=dict(
                    batch_size=32,
                    shuffle=True,
                ),
                valid=dict(
                    batch_size=1,
                ),
                test=dict(
                    batch_size=1,
                ),
            ),
            build_upstream=dict(
                name="fbank",
            ),
            build_featurizer=dict(
                layer_selections=None,
                normalize=False,
            ),
            build_downstream=dict(
                model_conf=dict(
                    module="LSTM",
                    proj_size=1024,
                    hidden_size=[1024, 1024],
                    dropout=[0.2, 0.2],
                    layer_norm=[False, False],
                    proj=[False, False],
                    sample_rate=[1, 1],
                    sample_style="concat",
                    bidirectional=True,
                ),
                specaug_conf=dict(
                    freq_mask_width_range=(0, 50),
                    num_freq_mask=4,
                    time_mask_width_range=(0, 40),
                    num_time_mask=2,
                ),
            ),
            build_model=dict(
                upstream_trainable=False,
            ),
            build_task=dict(
                log_metrics=[
                    "wer",
                    "cer",
                    "slot_type_f1",
                    "slot_value_cer",
                    "slot_value_wer",
                    "slot_edit_f1_full",
                    "slot_edit_f1_part",
                ],
            ),
            build_optimizer=dict(
                name="Adam",
                conf=dict(
                    lr=1.0e-4,
                ),
            ),
            build_scheduler=dict(
                name="ExponentialLR",
                gamma=0.9,
            ),
            save_model=dict(),
            save_task=dict(),
            train=dict(
                total_steps=200000,
                log_step=100,
                eval_step=2000,
                save_step=500,
                gradient_clipping=1.0,
                gradient_accumulate=1,
                valid_metric="slot_type_f1",
                valid_higher_better=True,
                auto_resume=True,
                resume_ckpt_dir=None,
            ),
        )

    def prepare_data(
        self,
        prepare_data: dict,
        target_dir: str,
        cache_dir: str,
        get_path_only: bool = False,
    ):
        """
        Prepare the task-specific data metadata (path, labels...).
        By default call :obj:`audio_snips_for_slot_filling` with :code:`**prepare_data`

        Args:
            prepare_data (dict): same in :obj:`default_config`, support arguments in :obj:`audio_snips_for_slot_filling`
            target_dir (str): Parse your corpus and save the csv file into this directory
            cache_dir (str): If the parsing or preprocessing takes too long time, you can save
                the temporary files into this directory. This directory is expected to be shared
                across different training sessions (different hypers and :code:`target_dir`)
            get_path_only (str): Directly return the filepaths no matter they exist or not.

        Returns:
            tuple

            1. train_path (str)
            2. valid_path (str)
            3. test_paths (List[str])

            Each path (str) should be a csv file containing the following columns:

            ====================  ====================
            column                description
            ====================  ====================
            id                    (str) - the unique id for this data point
            wav_path              (str) - the absolute path of the waveform file
            transcription         (str) - a text string
            ====================  ====================
        """
        return audio_snips_for_slot_filling(
            **self._get_current_arguments(flatten_dict="prepare_data")
        )

    def build_batch_sampler(
        self,
        build_batch_sampler: dict,
        target_dir: str,
        cache_dir: str,
        mode: str,
        data_csv: str,
        dataset,
    ):
        """
        Return the batch sampler for torch DataLoader.

        Args:
            build_batch_sampler (dict): same in :obj:`default_config`

                ====================  ====================
                key                   description
                ====================  ====================
                train                 (dict) - arguments for :obj:`FixedBatchSizeBatchSampler`
                valid                 (dict) - arguments for :obj:`FixedBatchSizeBatchSampler`
                test                  (dict) - arguments for :obj:`FixedBatchSizeBatchSampler`
                ====================  ====================

            target_dir (str): Current experiment directory
            cache_dir (str): If the preprocessing takes too long time, save
                the temporary files into this directory. This directory is expected to be shared
                across different training sessions (different hypers and :code:`target_dir`)
            mode (str): train/valid/test
            data_csv (str): the :code:`mode` specific csv from :obj:`prepare_data`
            dataset: the dataset from :obj:`build_dataset`

        Returns:
            batch sampler for torch DataLoader
        """

        @dataclass
        class Config:
            train: dict = None
            valid: dict = None
            test: dict = None

        conf = Config(**build_batch_sampler)

        if mode == "train":
            return FixedBatchSizeBatchSampler(dataset, **(conf.train or {}))
        elif mode == "valid":
            return FixedBatchSizeBatchSampler(dataset, **(conf.valid or {}))
        elif mode == "test":
            return FixedBatchSizeBatchSampler(dataset, **(conf.test or {}))
        else:
            raise ValueError(f"Unsupported mode: {mode}")



"""
Parse the Google Speech Commands V1 corpus

Authors:
  * Shu-wen Yang 2022
"""

import hashlib
import logging
import os
import re
from collections import OrderedDict
from pathlib import Path
from typing import List, Tuple, Union

from .base import Corpus

logger = logging.getLogger(__name__)

CLASSES = [
    "yes",
    "no",
    "up",
    "down",
    "left",
    "right",
    "on",
    "off",
    "stop",
    "go",
    "_unknown_",
    "_silence_",
]

__all__ = [
    "SpeechCommandsV1",
]


class SpeechCommandsV1(Corpus):
    """
    Args:
        dataset_root (str): should contain a 'dev' sub-folder for the training/validation set
            and a 'test' sub-folder for the testing set
    """

    def __init__(self, gsc1: str, gsc1_test: str, n_jobs: int = 4) -> None:
        train_dataset_root = Path(gsc1)
        test_dataset_root = Path(gsc1_test)

        train_list, valid_list = self.split_dataset(train_dataset_root)
        train_list = self.parse_train_valid_data_list(train_list, train_dataset_root)
        valid_list = self.parse_train_valid_data_list(valid_list, train_dataset_root)
        test_list = self.parse_test_data_list(test_dataset_root)

        self.train = self.list_to_dict(train_list)
        self.valid = self.list_to_dict(valid_list)
        self.test = self.list_to_dict(test_list)

        self._data = OrderedDict()
        self._data.update(self.train)
        self._data.update(self.valid)
        self._data.update(self.test)

    @staticmethod
    def split_dataset(
        root_dir: Union[str, Path], max_uttr_per_class=2**27 - 1
    ) -> Tuple[List[Tuple[str, str]], List[Tuple[str, str]]]:
        """Split Speech Commands into 3 set.

        Args:
            root_dir: speech commands dataset root dir
            max_uttr_per_class: predefined value in the original paper

        Return:
            train_list: [(class_name, audio_path), ...]
            valid_list: as above
        """
        train_list, valid_list = [], []

        for entry in Path(root_dir).iterdir():
            if not entry.is_dir() or entry.name == "_background_noise_":
                continue

            for audio_path in entry.glob("*.wav"):
                speaker_hashed = re.sub(r"_nohash_.*$", "", audio_path.name)
                hashed_again = hashlib.sha1(speaker_hashed.encode("utf-8")).hexdigest()
                percentage_hash = (int(hashed_again, 16) % (max_uttr_per_class + 1)) * (
                    100.0 / max_uttr_per_class
                )

                if percentage_hash < 10:
                    valid_list.append((entry.name, audio_path))
                elif percentage_hash < 20:
                    pass  # testing set is discarded
                else:
                    train_list.append((entry.name, audio_path))

        return train_list, valid_list

    @staticmethod
    def parse_train_valid_data_list(data_list, train_dataset_root: Path):
        data = [
            (class_name, audio_path)
            if class_name in CLASSES
            else ("_unknown_", audio_path)
            for class_name, audio_path in data_list
        ]
        data += [
            ("_silence_", audio_path)
            for audio_path in Path(train_dataset_root, "_background_noise_").glob(
                "*.wav"
            )
        ]
        return data

    @staticmethod
    def parse_test_data_list(test_dataset_root: Path):
        data = [
            (class_dir.name, audio_path)
            for class_dir in Path(test_dataset_root).iterdir()
            if class_dir.is_dir()
            for audio_path in class_dir.glob("*.wav")
        ]
        return data

    @staticmethod
    def path_to_unique_name(path: str):
        return "/".join(Path(path).parts[-2:])

    @classmethod
    def list_to_dict(cls, data_list):
        data = dict(
            {
                cls.path_to_unique_name(audio_path): {
                    "wav_path": audio_path,
                    "class_name": class_name,
                }
                for class_name, audio_path in data_list
            }
        )
        return data

    @property
    def all_data(self):
        """
        Return:
            Container: id (str)
                wav_path (str)
                class_name (str)
        """
        return self._data

    @property
    def data_split_ids(self):
        return list(self.train.keys()), list(self.valid.keys()), list(self.test.keys())

    @classmethod
    def download_dataset(cls, tgt_dir: str) -> None:
        import os
        import tarfile

        import requests

        assert os.path.exists(
            os.path.abspath(tgt_dir)
        ), "Target directory does not exist"

        def unzip_targz_then_delete(filepath: str, filename: str):
            file_path = os.path.join(
                os.path.abspath(tgt_dir), "CORPORA_DIR", filename.replace(".tar.gz", "")
            )
            os.makedirs(file_path)

            with tarfile.open(os.path.abspath(filepath)) as tar:
                tar.extractall(path=os.path.abspath(file_path))
            os.remove(os.path.abspath(filepath))

        def download_from_url(url: str):
            filename = url.split("/")[-1].replace(" ", "_")
            filepath = os.path.join(tgt_dir, filename)

            r = requests.get(url, stream=True)
            if r.ok:
                logger.info(f"Saving {filename} to", os.path.abspath(filepath))
                with open(filepath, "wb") as f:
                    for chunk in r.iter_content(chunk_size=1024 * 1024 * 10):
                        if chunk:
                            f.write(chunk)
                            f.flush()
                            os.fsync(f.fileno())
                logger.info(f"{filename} successfully downloaded")
                unzip_targz_then_delete(filepath, filename)
            else:
                logger.info(f"Download failed: status code {r.status_code}\n{r.text}")

        if not os.path.exists(
            os.path.join(os.path.abspath(tgt_dir), "CORPORA_DIR/speech_commands_v0.01")
        ):
            download_from_url(
                "http://download.tensorflow.org/data/speech_commands_v0.01.tar.gz"
            )
        if not os.path.exists(
            os.path.join(
                os.path.abspath(tgt_dir), "CORPORA_DIR/speech_commands_test_set_v0.01"
            )
        ):
            download_from_url(
                "http://download.tensorflow.org/data/speech_commands_test_set_v0.01.tar.gz"
            )
        logger.info(
            f"Speech commands dataset downloaded. Located at {os.path.abspath(tgt_dir)}/CORPORA_DIR/"
        )



import logging
import os
import tempfile
from collections import OrderedDict

import pytest
from dotenv import dotenv_values

from s3prl.dataio.corpus.librispeech import LibriSpeech
from s3prl.dataset.base import AugmentedDynamicItemDataset
from s3prl.dataset.speech2text_pipe import Speech2TextPipe


@pytest.mark.corpus
def test_speech2text_pipe():
    config = dotenv_values()

    train_data, valid_data, test_data = LibriSpeech(config["LibriSpeech"]).data_split
    train_data = OrderedDict(train_data)

    dataset = AugmentedDynamicItemDataset(train_data)
    dataset_char = Speech2TextPipe(generate_tokenizer=True, vocab_type="character")(
        dataset
    )

    assert dataset_char.get_tool("output_size") == 31

    with tempfile.TemporaryDirectory() as directory:
        logging.info(directory)
        output_file = os.path.join(directory, "subword")
        dataset = AugmentedDynamicItemDataset(train_data)
        dataset_subword = Speech2TextPipe(
            generate_tokenizer=True,
            vocab_type="subword",
            vocab_args={"vocab_size": 4000, "output_file": output_file},
        )(dataset)

        assert dataset_subword.get_tool("output_size") == 4000



import torch
from dataclasses import dataclass
import pickle
from collections import OrderedDict, defaultdict
import pandas as pd
import json
from pathlib import Path
from s3prl.nn.hear import HearFullyConnectedPrediction
from s3prl.task.scene_prediction import ScenePredictionTask
from s3prl.dataio.encoder import CategoryEncoder
from s3prl.dataio.sampler import FixedBatchSizeBatchSampler
from s3prl.dataset.utterance_classification_pipe import HearScenePipe

from omegaconf import MISSING

from .superb_sid import SuperbSID


def hear_scene_trainvaltest(
    target_dir: str,
    cache_dir: str,
    dataset_root: str,
    get_path_only: bool = False,
):
    target_dir = Path(target_dir)
    dataset_root = Path(dataset_root)
    wav_root: Path = dataset_root / "16000"

    train_csv = target_dir / "train.csv"
    valid_csv = target_dir / "valid.csv"
    test_csv = target_dir / "test_csv"

    if get_path_only:
        return train_csv, valid_csv, [test_csv]

    def load_json(filepath):
        with open(filepath, "r") as fp:
            return json.load(fp)

    def split_to_df(split: str) -> pd.DataFrame:
        meta = load_json(dataset_root / f"{split}.json")
        data = defaultdict(list)
        for k in list(meta.keys()):
            data["id"].append(k)
            data["wav_path"].append(wav_root / split / k)
            data["labels"].append(",".join([str(label).strip() for label in meta[k]]))
        return pd.DataFrame(data=data)

    split_to_df("train").to_csv(train_csv, index=False)
    split_to_df("valid").to_csv(valid_csv, index=False)
    split_to_df("test").to_csv(test_csv, index=False)

    return train_csv, valid_csv, [test_csv]


class HearFSD(SuperbSID):
    def default_config(self) -> dict:
        return dict(
            start=0,
            stop=None,
            target_dir=MISSING,
            cache_dir=None,
            remove_all_cache=False,
            prepare_data=dict(
                dataset_root=MISSING,
            ),
            build_batch_sampler=dict(
                train=dict(
                    batch_size=10,
                    shuffle=True,
                ),
                valid=dict(
                    batch_size=1,
                ),
                test=dict(
                    batch_size=1,
                ),
            ),
            build_upstream=dict(
                name="fbank",
            ),
            build_featurizer=dict(
                layer_selections=None,
                normalize=False,
            ),
            build_downstream=dict(
                hidden_layers=2,
                pooling_type="MeanPooling",
            ),
            build_model=dict(
                upstream_trainable=False,
            ),
            build_task=dict(
                prediction_type="multilabel",
                scores=["mAP", "top1_acc", "d_prime", "aucroc"],
            ),
            build_optimizer=dict(
                name="Adam",
                conf=dict(
                    lr=1.0e-4,
                ),
            ),
            build_scheduler=dict(
                name="ExponentialLR",
                gamma=0.9,
            ),
            save_model=dict(),
            save_task=dict(),
            train=dict(
                total_steps=40000,
                log_step=100,
                eval_step=1000,
                save_step=100,
                gradient_clipping=1.0,
                gradient_accumulate=4,
                valid_metric="mAP",
                valid_higher_better=True,
                auto_resume=True,
                resume_ckpt_dir=None,
            ),
            evaluate=dict(),
        )

    def prepare_data(
        self,
        prepare_data: dict,
        target_dir: str,
        cache_dir: str,
        get_path_only: bool = False,
    ):
        return hear_scene_trainvaltest(
            **self._get_current_arguments(flatten_dict="prepare_data")
        )

    def build_encoder(
        self,
        build_encoder: dict,
        target_dir: str,
        cache_dir: str,
        train_csv_path: str,
        valid_csv_path: str,
        test_csv_paths: list,
        get_path_only: bool = False,
    ):
        encoder_path = Path(target_dir) / "encoder.pkl"
        if get_path_only:
            return encoder_path

        train_csv = pd.read_csv(train_csv_path)
        valid_csv = pd.read_csv(valid_csv_path)
        test_csvs = [pd.read_csv(path) for path in test_csv_paths]
        all_csv = pd.concat([train_csv, valid_csv, *test_csvs])
        all_labels = []
        for rowid, row in all_csv.iterrows():
            labels = str(row["labels"]).split(",")
            labels = [l.strip() for l in labels]
            all_labels.extend(labels)

        encoder = CategoryEncoder(all_labels)
        with open(encoder_path, "wb") as f:
            pickle.dump(encoder, f)

        return encoder_path

    def build_dataset(
        self,
        build_dataset: dict,
        target_dir: str,
        cache_dir: str,
        mode: str,
        data_csv: str,
        encoder_path: str,
        frame_shift: int,
    ):
        df = pd.read_csv(data_csv)
        data = OrderedDict()
        for rowid, row in df.iterrows():
            data[row["id"]] = dict(
                wav_path=row["wav_path"],
                labels=[label.strip() for label in str(row["labels"]).split(",")],
            )

        with open(encoder_path, "rb") as f:
            encoder = pickle.load(f)

        dataset = HearScenePipe()(data, tools={"category": encoder})
        return dataset

    def build_batch_sampler(
        self,
        build_batch_sampler: dict,
        target_dir: str,
        cache_dir: str,
        mode: str,
        data_csv: str,
        dataset,
    ):
        @dataclass
        class Config:
            train: dict = None
            valid: dict = None
            test: dict = None

        conf = Config(**build_batch_sampler)
        return FixedBatchSizeBatchSampler(dataset, **(conf.train or {}))

    def build_downstream(
        self,
        build_downstream: dict,
        downstream_input_size: int,
        downstream_output_size: int,
        downstream_input_stride: int,
    ):
        return HearFullyConnectedPrediction(
            downstream_input_size, downstream_output_size, **build_downstream
        )

    def build_task(
        self,
        build_task: dict,
        model: torch.nn.Module,
        encoder,
        valid_df: pd.DataFrame = None,
        test_df: pd.DataFrame = None,
    ):
        return ScenePredictionTask(model, encoder, **build_task)



# -*- coding: utf-8 -*-
# @Time    : 8/25/21 5:25 PM
# @Author  : Yuan Gong
# @Affiliation  : Massachusetts Institute of Technology
# @Email   : yuangong@mit.edu
# @File    : expert.py

import torch
import logging

from s3prl.upstream.interfaces import SAMPLE_RATE

from ..interfaces import SAMPLE_RATE
from ..ssast.audio import FeatureExtractor

logger = logging.getLogger(__name__)
STRIDE = 160


class UpstreamExpert(torch.nn.Module):
    def __init__(self, ckpt: str, segment_secs: float = 10.24):
        super().__init__()
        self.segment_secs = segment_secs
        target_length = int(segment_secs * SAMPLE_RATE / STRIDE)

        try:
            import timm
            from .ast_models import ASTModel
        except:
            print(
                "SSAST requires 'timm==0.4.5' to work. Please run 'pip install timm==0.4.5'"
            )
            raise

        self.preprocessor = FeatureExtractor(
            target_length=target_length, apply_cmvn=False
        )
        self.model = ASTModel(
            label_dim=527,
            fstride=10,
            tstride=10,
            input_fdim=128,
            input_tdim=1024,
            imagenet_pretrain=True,
            audioset_pretrain=False,
            model_size="base384",
            pretrained_ckpt=ckpt,
        )

    def get_downsample_rates(self, key: str = None) -> int:
        return int(self.segment_secs * SAMPLE_RATE)

    def forward(self, wavs):
        wavs_len = [len(wav) for wav in wavs]
        max_wav_len = max(wavs_len)
        seg_n_sample = int(self.segment_secs * SAMPLE_RATE)
        padded_max_wav_len = (max_wav_len // seg_n_sample + 1) * seg_n_sample
        padded_wavs = [
            torch.cat([wav, wav.new_zeros(padded_max_wav_len - len(wav))])
            for wav in wavs
        ]

        all_ts = []
        for start in range(0, max(len(wav) for wav in padded_wavs), seg_n_sample):
            subwavs = [wav[start : start + seg_n_sample] for wav in padded_wavs]
            features = [self.preprocessor(wav.unsqueeze(0)) for wav in subwavs]
            features = torch.stack(features, dim=0)
            output = self.model(features)  # (batch_size, hidden_size)
            all_ts.append(output)

        all_hs = torch.stack(all_ts, dim=1)
        max_feat_len = int(max_wav_len // self.get_downsample_rates() + 1)
        all_hs = all_hs[:, :max_feat_len, :].float()

        return {"hidden_states": [all_hs]}



from tqdm import tqdm
from pathlib import Path
from collections import defaultdict
from os.path import join, getsize, isfile
from joblib import Parallel, delayed
from torch.utils.data import Dataset


def parse_lexicon(line, tokenizer):
    line.replace('\t', ' ')
    word, *phonemes = line.split()
    for p in phonemes:
        assert p in tokenizer._vocab2idx.keys()
    return word, phonemes


def read_text(file, word2phonemes, tokenizer):
    '''Get transcription of target wave file, 
       it's somewhat redundant for accessing each txt multiplt times,
       but it works fine with multi-thread'''
    src_file = '-'.join(file.split('-')[:-1])+'.trans.txt'
    idx = file.split('/')[-1].split('.')[0]

    with open(src_file, 'r') as fp:
        for line in fp:
            if idx == line.split(' ')[0]:
                transcription = line[:-1].split(' ', 1)[1]
                phonemes = []
                for word in transcription.split():
                    phonemes += word2phonemes[word]
                return tokenizer.encode(' '.join(phonemes))


class LibriPhoneDataset(Dataset):
    def __init__(self, split, tokenizer, bucket_size, path, lexicon, ascending=False, **kwargs):
        # Setup
        self.path = path
        self.bucket_size = bucket_size

        # create word -> phonemes mapping
        word2phonemes_all = defaultdict(list)
        for lexicon_file in lexicon:
            with open(lexicon_file, 'r') as file:
                lines = [line.strip() for line in file.readlines()]
                for line in lines:
                    word, phonemes = parse_lexicon(line, tokenizer)
                    word2phonemes_all[word].append(phonemes)

        # check mapping number of each word
        word2phonemes = {}
        for word, phonemes_all in word2phonemes_all.items():
            if len(phonemes_all) > 1:
                print(f'[LibriPhone] - {len(phonemes_all)} of phoneme sequences found for {word}.')
                for idx, phonemes in enumerate(phonemes_all):
                    print(f'{idx}. {phonemes}')
            word2phonemes[word] = phonemes_all[0]
        print(f'[LibriPhone] - Taking the first phoneme sequences for a deterministic behavior.')

        # List all wave files
        file_list = []
        for s in split:
            split_list = list(Path(join(path, s)).rglob("*.flac"))
            assert len(split_list) > 0, "No data found @ {}".format(join(path,s))
            file_list += split_list
        
        text = []
        for f in tqdm(file_list, desc='word -> phonemes'):
            text.append(read_text(str(f), word2phonemes, tokenizer))

        self.file_list, self.text = zip(*[(f_name, txt)
                                          for f_name, txt in sorted(zip(file_list, text), reverse=not ascending, key=lambda x:len(x[1]))])
    
    def __getitem__(self, index):
        if self.bucket_size > 1:
            index = min(len(self.file_list)-self.bucket_size, index)
            return [(f_path, txt) for f_path, txt in
                    zip(self.file_list[index:index+self.bucket_size], self.text[index:index+self.bucket_size])]
        else:
            return self.file_list[index], self.text[index]

    def __len__(self):
        return len(self.file_list)



import os
import argparse
import pandas as pd
import numpy as np


def get_preprocess_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--npy_path', default='../../data/mosei/mel160', type=str, help='Path to MOSEI segmented NPY files')
    parser.add_argument('--csv_path', default='../../data/mosei/mosei_no_semi.csv', type=str, help='Path to mosei_no_semi.csv', required=False)
    args = parser.parse_args()
    return args


def add_length(args):
    csv = pd.read_csv(args.csv_path)
    lengths = []
    for index, row in csv.iterrows():
        npy = np.load(os.path.join(args.npy_path, row.key + '.npy'))
        lengths.append(npy.shape[0])
    csv['length'] = lengths
    csv.to_csv(args.csv_path, index=False)


def main():
    # get arguments
    args = get_preprocess_args()

    # Acoustic Feature Extraction
    add_length(args)


if __name__ == '__main__':
    main()




# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ preprocess_alignment.py ]
#   Synopsis     [ preprocess phone alignment for the LibriSpeech dataset ]
#   Author       [ Andy T. Liu (Andi611) ]
#   Copyright    [ Copyleft(c), Speech Lab, NTU, Taiwan ]
#   Reference    [ https://github.com/BogiHsu/Phone-Recognizer/blob/815cf9375045c053fa57d17fad0fa14fdc3c7bee/loader.py#L28 ]
"""*********************************************************************************************"""


###############
# IMPORTATION #
###############
import os
import pickle
import argparse
import numpy as np
import pandas as pd
from tqdm import tqdm
from pathlib import Path
from utility.audio import sample_rate, _stft_parameters


#############################
# PREPROCESS CONFIGURATIONS #
#############################
def get_preprocess_args():
    
    parser = argparse.ArgumentParser(description='preprocess arguments for LibriSpeech dataset.')
    parser.add_argument('--data_path', default='./data/libri_alignment', type=str, help='Path to raw LibriSpeech alignment')
    parser.add_argument('--output_path', default='./data/libri_phone', type=str, help='Path to store output', required=False)
    args = parser.parse_args()
    return args


####################
# PHONE PREPROCESS #
####################
def phone_preprocess(data_path, output_path, sets, unaligned):
    
    print('Data sets :')
    for idx, s in enumerate(sets):
        print('\t', idx, ':', s)
    todo_sets = input('Please enter the index for preprocessing sets (seperate w/ space): ')
    sets = [sets[int(s)] for s in todo_sets.split(' ')]
    
    # compute phone2idx
    idx = 0
    phone2idx = {}
    for s in sets:
        print('')
        print('Computing', s, 'data...')
        for path in tqdm(list(Path(os.path.join(data_path, s)).rglob("*.txt"))):
            check_name = path.as_posix().split('/')[-1].split('.')[0]
            if check_name not in unaligned and check_name != 'unaligned': # ignore the unaligned files and `unaligned.txt` itself
                for line in open(path).readlines():
                    phone = line.strip('\n').split(' ')[-1]
                    if phone not in phone2idx:
                        phone2idx[phone] = idx
                        idx += 1
    print('Phone set:')
    print(phone2idx)
    print(len(phone2idx), 'distinct phones found in', sets)
    with open(os.path.join(output_path, 'phone2idx.pkl'), "wb") as fp:
            pickle.dump(phone2idx, fp)

    for s in sets:
        print('')
        print('Preprocessing', s, 'data...')
        todo = list(Path(os.path.join(data_path, s)).rglob("*.txt"))
        print(len(todo),'audio files found in', s)
        if not os.path.exists(os.path.join(output_path, s)): 
            os.makedirs(os.path.join(output_path, s))

        print('Preprocessing phone alignments...', flush=True)
        for path in tqdm(todo):
            check_name = path.as_posix().split('/')[-1].split('.')[0]
            if check_name not in unaligned and check_name != 'unaligned': # ignore the unaligned files and `unaligned.txt` itself
                x = []
                file = open(path).readlines()
                for line in file:
                    line = line.strip('\n').split(' ')
                    x += time_to_frame(start_time=float(line[0]), end_time=float(line[1]), phone=phone2idx[line[2]])
                x = np.asarray(x)
                path_to_save = str(path).replace(data_path.split('/')[-1], output_path.split('/')[-1]).replace('txt', 'pkl')
                with open(path_to_save, "wb") as fp:
                    pickle.dump(x, fp)

    print('Phone preprocessing complete!')      


#################
# TIME TO FRAME #
#################
def time_to_frame(start_time, end_time, phone):
    phones = []
    
    start_time = int(start_time * sample_rate)
    end_time = int(end_time * sample_rate)
    
    _, hop_length, win_length = _stft_parameters(sample_rate=sample_rate)
    h_window = win_length * 0.5 # select the middle of a window

    start_time = (start_time - h_window) if start_time >= h_window else 0
    end_time = (end_time - h_window) if end_time >= h_window else 0
    times = (end_time // hop_length) - (start_time // hop_length) \
            + (1 if start_time % hop_length == 0 else 0) - (1 if end_time % hop_length == 0 else 0)
    phones += [phone] * int(times)
    return phones


########
# MAIN #
########
def main():

    # get arguments
    args = get_preprocess_args()
    
    # mkdir
    if not os.path.exists(args.output_path):
        os.makedirs(args.output_path)

    # dump unaligned text
    try:
        file = open(os.path.join(args.data_path, 'train-clean-360/unaligned.txt')).readlines()
        unaligned = [str(line).split('\t')[0].split(' ')[0] for line in file]
        print('Unaligned list: ', unaligned)
        unaligned_pkl = ['train-clean-360/' + u + '.npy' for u in unaligned]
        with open(os.path.join(args.output_path, 'unaligned.pkl'), "wb") as fp:
            pickle.dump(unaligned_pkl, fp)
    except:
        raise ValueError('Did not find unaligned.txt!')

    # Process data
    sets = ['train-clean-360', 'test-clean'] # only two sets available for now
    # sets = ['train-clean-100','train-clean-360','train-other-500','dev-clean','dev-other','test-clean','test-other']
    phone_preprocess(args.data_path, args.output_path, sets, unaligned)


if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ preprocess_mosi.py ]
#   Synopsis     [ preprocessing for the MOSI dataset ]
#   Author       [ Andy T. Liu (Andi611) ]
#   Copyright    [ Copyleft(c), Speech Lab, NTU, Taiwan ]
"""*********************************************************************************************"""


###############
# IMPORTATION #
###############
import os
import shutil
import glob
import sys
import pickle
import argparse
import numpy as np
import pandas as pd
from tqdm import tqdm
from pathlib import Path
from joblib import Parallel, delayed
from utility.asr import encode_target
from utility.audio import extract_feature, mel_dim, num_freq

# using CMU SDK to access audio label
# for installing mmsdk package, please follow the instructions in:
# https://github.com/A2Zadeh/CMU-MultimodalSDK#installation
import mmsdk
from mmsdk import mmdatasdk as md


##################
# BOOLEAB STRING #
##################
def boolean_string(s):
    if s not in ['False', 'True']:
        raise ValueError('Not a valid boolean string')
    return s == 'True'

def sdk2npy(string):
    # not data type conversion
    # convert the name format in CMU sdk to corresponding npy file name
    split = string.split('[')
    utterance_name = split[0]
    number = int(split[1].split(']')[0])
    string = utterance_name + '_' + str(number + 1) + '.npy'
    return string

def npy2sdk(string):
    # not data type conversion
    # inverse operation of sdk2npy
    split = string.split('_')
    number = int(split[-1][:-4])
    utterance_name = '_'.join(split[:-1])
    string = utterance_name + '[' + str(number - 1) + ']'
    return string


#############################
# PREPROCESS CONFIGURATIONS #
#############################
def get_preprocess_args():
    parser = argparse.ArgumentParser(description='preprocess arguments for LibriSpeech dataset.')
    parser.add_argument('--data_path', default='/home/leo/d/datasets/MOSI/Raw/Audio/WAV_16000/Segmented', type=str, help='Path to raw MOSI segmented audio dataset')
    parser.add_argument('--output_path', default='../data/', type=str, help='Path to store output', required=False)
    parser.add_argument('--feature_type', default='mel', type=str, help='Feature type ( mfcc / fbank / mel / linear )', required=False)
    parser.add_argument('--apply_cmvn', default=True, type=boolean_string, help='Apply CMVN on feature', required=False)
    parser.add_argument('--n_jobs', default=-1, type=int, help='Number of jobs used for feature extraction', required=False)
    parser.add_argument('--n_tokens', default=5000, type=int, help='Vocabulary size of target', required=False)
    args = parser.parse_args()
    return args


#######################
# ACOUSTIC PREPROCESS #
#######################
def acoustic_preprocess(args, dim):
    # Extracting features from audio
    todo = list(Path(args.data_path).glob("*.wav"))
    print(len(todo),'audio files found in MOSI')

    assert args.feature_type in ['mel', 'linear', 'fbank'], 'Feature type unsupported'
    output_dir = os.path.join(args.output_path,'_'.join(['mosi',str(args.feature_type)+str(dim)]))
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    print('Extracting acoustic feature...',flush=True)
    tr_x = Parallel(n_jobs=args.n_jobs)(delayed(extract_feature)(str(file), feature=args.feature_type, cmvn=args.apply_cmvn, \
                                        save_feature=os.path.join(output_dir, str(file).split('/')[-1].replace('.wav',''))) for file in tqdm(todo))

    # Loading labels
    DATASET = md.cmu_mosi
    try:
        md.mmdataset(DATASET.labels, args.data_path)
    except RuntimeError:
        print("Labels have been downloaded previously.")

    label_field = 'CMU_MOSI_Opinion_Labels'
    features = [
        label_field,
    ]

    recipe = {feat: os.path.join(args.data_path, feat) + '.csd' for feat in features}
    dataset = md.mmdataset(recipe)
    dataset.align(label_field)

    # Check each label has corresponding utterance
    utterances = os.listdir(output_dir)
    for segment_sdk in dataset[label_field].keys():
        segment_npy = sdk2npy(segment_sdk)
        try:
            assert segment_npy in utterances
        except AssertionError:
            print('AssertionError: Cannot find corresponding utterance for given label')

        # Check the inverse is itself by npy2sdk
        try:
            assert npy2sdk(segment_npy) == segment_sdk
        except AssertionError:
            print('AssertionError: npt2sdk funtion has bug')

    # sort by len
    sorted_xlen = []
    sorted_y = []
    sorted_todo = []
    for idx in reversed(np.argsort(tr_x)):
        filename = str(todo[idx]).split('/')[-1].replace('.wav','.npy')
        sdkname = npy2sdk(filename)
        if sdkname in dataset[label_field].keys():
            sorted_xlen.append(tr_x[idx])
            sorted_y.append(dataset[label_field][sdkname]['features'].reshape(-1)[0])
            sorted_todo.append(filename)

    # Organize label
    df = pd.DataFrame(data={'file_path':[fp for fp in sorted_todo],'length':list(sorted_xlen),'label':sorted_y})

    # Split data into train/dev/test
    train_split = DATASET.standard_folds.standard_train_fold
    dev_split = DATASET.standard_folds.standard_valid_fold
    test_split = DATASET.standard_folds.standard_test_fold

    npy_dir = os.path.join(output_dir, 'npy')
    if not os.path.exists(npy_dir):
        os.mkdir(npy_dir)

    def classify(file_name):
        file_name = file_name[0]
        prefix = '_'.join(file_name.split('_')[:-1])
        shutil.move(os.path.join(output_dir,file_name), os.path.join(npy_dir, file_name))
        if prefix in train_split:
            return 'train'
        elif prefix in dev_split:
            return 'dev'
        elif prefix in test_split:
            return 'test'
        else: assert 0, 'Error in preprocess_mosi.py:146'
    
    belong = np.apply_along_axis(lambda file_name: classify(file_name), 1, df['file_path'].values.reshape(-1, 1))
    df.insert(len(df.columns), 'set', belong)
    train_frame = df[df.set == 'train']
    dev_frame = df[df.set == 'dev']
    test_frame = df[df.set == 'test']
    
    # dump csv of all splits
    df.to_csv(os.path.join(output_dir,'all.csv'))
    train_frame.to_csv(os.path.join(output_dir,'train.csv'))
    dev_frame.to_csv(os.path.join(output_dir,'dev.csv'))
    test_frame.to_csv(os.path.join(output_dir,'test.csv'))

    remain_npy = glob.glob(os.path.join(output_dir, '*.npy'))
    print('Delete ' + str(len(remain_npy)) + ' unlabeled npy files:')
    for npy in remain_npy:
        print('delete ' + npy)
        os.remove(npy)
    print('All done, saved at', output_dir, 'exit.')


########
# MAIN #
########
def main():
    # get arguments
    args = get_preprocess_args()
    dim = num_freq if args.feature_type == 'linear' else mel_dim

    # Acoustic Feature Extraction
    acoustic_preprocess(args, dim)


if __name__ == '__main__':
    main()

    



# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ upstream/byol_a/hubconf.py ]
#   Synopsis     [ the BYOL-A torch hubconf ]
#   Author       [ S3PRL ]
#   Copyright    [ Copyleft(c), Speech Lab, NTU, Taiwan ]
"""*********************************************************************************************"""

import os

from s3prl.util.download import _urls_to_filepaths

from .expert import UpstreamExpert as _UpstreamExpert

# FIXME (Leo): These are wrong according to the original author


def byol_a_local(ckpt, model_config=None, *args, **kwargs):
    """
    The model from local ckpt
        ckpt (str): PATH
    """
    assert os.path.isfile(ckpt)
    if model_config is not None:
        assert os.path.isfile(model_config)
    return _UpstreamExpert(ckpt, model_config, *args, **kwargs)


def byol_a_url(ckpt, refresh=False, *args, **kwargs):
    """
    The model from URL
        ckpt (str): URL
    """
    return byol_a_local(_urls_to_filepaths(ckpt, refresh=refresh), *args, **kwargs)


def byol_a(refresh=False, *args, **kwargs):
    """
    The default model
        refresh (bool): whether to download ckpt/config again if existed
    """
    return byol_a_2048(refresh=refresh, *args, **kwargs)


def byol_a_2048(refresh=False, *args, **kwargs):
    """
    refresh (bool): whether to download ckpt/config again if existed
    """
    kwargs[
        "ckpt"
    ] = "https://github.com/nttcslab/byol-a/raw/master/pretrained_weights/AudioNTT2020-BYOLA-64x96d2048.pth"
    return byol_a_url(refresh=refresh, *args, **kwargs)


def byol_a_1024(refresh=False, *args, **kwargs):
    """
    refresh (bool): whether to download ckpt/config again if existed
    """
    kwargs[
        "ckpt"
    ] = "https://github.com/nttcslab/byol-a/raw/master/pretrained_weights/AudioNTT2020-BYOLA-64x96d1024.pth"
    return byol_a_url(refresh=refresh, *args, **kwargs)


def byol_a_512(refresh=False, *args, **kwargs):
    """
    refresh (bool): whether to download ckpt/config again if existed
    """
    kwargs[
        "ckpt"
    ] = "https://github.com/nttcslab/byol-a/raw/master/pretrained_weights/AudioNTT2020-BYOLA-64x96d512.pth"
    return byol_a_url(refresh=refresh, *args, **kwargs)



# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ preprocess_timit.py ]
#   Synopsis     [ preprocess text transcripts and audio speech for the TIMIT dataset ]
#   Author       [ Andy T. Liu (Andi611) ]
#   Copyright    [ Copyleft(c), Speech Lab, NTU, Taiwan ]
#   Reference    [ https://github.com/Alexander-H-Liu/End-to-end-ASR-Pytorch ]
"""*********************************************************************************************"""


###############
# IMPORTATION #
###############
import os
import sys
import pickle
import argparse
import numpy as np
import pandas as pd
from tqdm import tqdm
from pathlib import Path
from joblib import Parallel, delayed
from utility.audio import extract_feature, num_mels, num_mfcc, num_freq


##################
# BOOLEAB STRING #
##################
def boolean_string(s):
    if s not in ['False', 'True']:
        raise ValueError('Not a valid boolean string')
    return s == 'True'


#############################
# PREPROCESS CONFIGURATIONS #
#############################
def get_preprocess_args():
    
    parser = argparse.ArgumentParser(description='preprocess arguments for LibriSpeech dataset.')

    parser.add_argument('--data_path', default='./data/timit', type=str, help='Path to raw TIMIT dataset')
    parser.add_argument('--output_path', default='./data/', type=str, help='Path to store output', required=False)

    parser.add_argument('--feature_type', default='mel', type=str, help='Feature type ( mfcc / fbank / mel / linear )', required=False)
    parser.add_argument('--delta', default=True, type=boolean_string, help='Append Delta', required=False)
    parser.add_argument('--delta_delta', default=False, type=boolean_string, help='Append Delta Delta', required=False)
    parser.add_argument('--apply_cmvn', default=True, type=boolean_string, help='Apply CMVN on feature', required=False)

    parser.add_argument('--n_jobs', default=-1, type=int, help='Number of jobs used for feature extraction', required=False)
    parser.add_argument('--name', default='None', type=str, help='Name of the output directory', required=False)

    args = parser.parse_args()
    return args


##############
# PREPROCESS #
##############
def preprocess(args, dim):
    for s in ('train', 'dev', 'test'):
        print('')
        print(f'Preprocessing {s} data...', end='')
        todo = list(Path(os.path.join(args.data_path, s.upper())).rglob("*.[wW][aA][vV]"))
        if len(todo) == 0: todo = list(Path(os.path.join(args.data_path, s)).rglob("*.[wW][aA][vV]"))
        print(len(todo), f'audio files found in {s} set')

        if args.name == 'None':
            output_dir = os.path.join(args.output_path, '_'.join(['timit', str(args.feature_type)+str(dim)]))
        else:
            output_dir = os.path.join(args.output_path, args.name)
        if not os.path.exists(output_dir): os.makedirs(output_dir)
        cur_path = os.path.join(output_dir, s)
        if not os.path.exists(cur_path): os.makedirs(cur_path)


        print('Extracting acoustic feature...', flush=True)
        tr_x = Parallel(n_jobs=args.n_jobs)(delayed(extract_feature)(str(file), feature=args.feature_type, \
                                            delta=args.delta, delta_delta=args.delta_delta, cmvn=args.apply_cmvn, \
                                            save_feature=os.path.join(cur_path, str(file).split('/')[-1].split('.')[0])) for file in tqdm(todo))

        
        # sort by len
        sorted_todo = [os.path.join(s, str(todo[idx]).split('/')[-1].split('.')[0]+'.npy') for idx in reversed(np.argsort(tr_x))]
        # Dump data
        df = pd.DataFrame(data={'file_path':[fp for fp in sorted_todo], 'length':list(reversed(sorted(tr_x))), 'label':None})
        df.to_csv(os.path.join(output_dir, s+'.csv'))

    print('All done, saved at', output_dir, 'exit.')


########
# MAIN #
########
def main():

    # get arguments
    args = get_preprocess_args()
    mel_dim = num_mels * (1 + int(args.delta) + int(args.delta_delta))
    mfcc_dim = num_mfcc * (1 + int(args.delta) + int(args.delta_delta))
    dim = num_freq if args.feature_type == 'linear' else (mfcc_dim if args.feature_type == 'mfcc' else mel_dim)
    print('Delta: ', args.delta, '. Delta Delta: ', args.delta_delta, '. Cmvn: ', args.apply_cmvn)

    # Process data
    preprocess(args, dim)


if __name__ == '__main__':
    main()


from tqdm import tqdm
from pathlib import Path
from os.path import join, getsize
from joblib import Parallel, delayed
from torch.utils.data import Dataset


def read_text(file):
    '''Get transcription of target wave file, 
       it's somewhat redundant for accessing each txt multiplt times,
       but it works fine with multi-thread'''
    src_file = '-'.join(file.split('-')[:-1])+'.trans.txt'
    idx = file.split('/')[-1].split('.')[0]

    with open(src_file, 'r') as fp:
        for line in fp:
            if idx == line.split(' ')[0]:
                return line[:-1].split(' ', 1)[1]


class LibriDataset(Dataset):
    def __init__(self, split, tokenizer, bucket_size, path, ascending=False, **kwargs):
        # Setup
        self.path = path
        self.bucket_size = bucket_size

        # List all wave files
        file_list = []
        for s in split:
            split_list = list(Path(join(path, s)).rglob("*.flac"))
            assert len(split_list) > 0, "No data found @ {}".format(join(path,s))
            file_list += split_list
        
        text = []
        for f in tqdm(file_list, desc='Read text'):
            transcription = read_text(str(f))
            text.append(tokenizer.encode(transcription))

        self.file_list, self.text = zip(*[(f_name, txt)
                                          for f_name, txt in sorted(zip(file_list, text), reverse=not ascending, key=lambda x:len(x[1]))])

    def __getitem__(self, index):
        if self.bucket_size > 1:
            # Return a bucket
            index = min(len(self.file_list)-self.bucket_size, index)
            return [(f_path, txt) for f_path, txt in
                    zip(self.file_list[index:index+self.bucket_size], self.text[index:index+self.bucket_size])]
        else:
            return self.file_list[index], self.text[index]

    def __len__(self):
        return len(self.file_list)



"""
Simple categorical encoder

Authors:
  Shu-wen Yang 2022
"""

from typing import List


class CategoryEncoder:
    def __init__(self, category: List[str]) -> None:
        self.category = list(sorted(set(category)))

    def __len__(self):
        return len(self.category)

    def encode(self, label):
        return self.category.index(label)

    def decode(self, index):
        return self.category[index]


class CategoryEncoders:
    def __init__(self, categories: List[List[str]]) -> None:
        self.categories = [CategoryEncoder(c) for c in categories]

    def __len__(self):
        return sum([len(c) for c in self.categories])

    def __iter__(self) -> List[CategoryEncoder]:
        for c in self.categories:
            yield c



# -*- coding: utf-8 -*-
# @Time    : 8/25/21 5:25 PM
# @Author  : Yuan Gong
# @Affiliation  : Massachusetts Institute of Technology
# @Email   : yuangong@mit.edu
# @File    : hubconf.py

from s3prl.util.download import _urls_to_filepaths
from .expert import UpstreamExpert as _UpstreamExpert

# Frame-based SSAST
# 1s for speech commands, 6s for IEMOCAP, 10s for SID
def ssast_frame_base(*args, segment_secs: float = 1.0, **kwargs):
    kwargs["model_size"] = "base_f"
    kwargs["pretrain_path"] = _urls_to_filepaths(
        "https://www.dropbox.com/s/nx6nl4d4bl71sm8/SSAST-Base-Frame-400.pth?dl=1"
    )
    kwargs["target_length"] = round(segment_secs * 100)
    return _UpstreamExpert(*args, **kwargs)


# FIXME: Enable Patch-based
# Patch-based SSAST
# 1s for speech commands, 6s for IEMOCAP, 10s for SID
# def ssast_patch_base(*args, mode: str = "1.0_sec", **kwargs):
#     kwargs["model_size"] = "base_p"
#     kwargs["pretrain_path"] = _urls_to_filepaths("https://www.dropbox.com/s/ewrzpco95n9jdz6/SSAST-Base-Patch-400.pth?dl=1")
#     kwargs["target_length"] = round(segment_secs * 100)
#     return _UpstreamExpert(*args, **kwargs)



#!/usr/bin/env python

# Data preparation code for the Libri2Mix dataset

import os
import argparse
import soundfile as sf
import shutil

parser = argparse.ArgumentParser(description='Prepare Libri2Mix dataset')
parser.add_argument('src_dir', type=str, help='Data directory to Libri2Mix')
parser.add_argument('tgt_dir', type=str, help='Target data directory')
parser.add_argument('--sample_rate', type=str, choices=['8k', '16k'], default='16k', help='Sample rate')
parser.add_argument('--mode', type=str, choices=['max', 'min'], default='min', help='Length mode')
parser.add_argument('--part', type=str, choices=['train-100', 'train-360', 'dev', 'test'], default='dev', help='Partition of dataset')
args = parser.parse_args()

def main():
    output_dir = "{}/wav{}/{}/{}".format(args.tgt_dir, args.sample_rate, args.mode, args.part)
    if os.path.exists(output_dir):
        raise ValueError("Warning: {} already exists, please check!")
    else:
        os.makedirs(output_dir)
    
    wav_dir = "{}/wav{}/{}/{}".format(args.src_dir, args.sample_rate, args.mode, args.part)
    assert os.path.exists(wav_dir)
    for cond in ["s1", "s2", "mix_clean", "mix_both", "mix_single", "noise"]:
        filelist = [f for f in os.listdir("{}/{}".format(wav_dir, cond)) if f.endswith(".wav")]
        filelist.sort()
        cond_dir = "{}/{}".format(output_dir, cond)
        if not os.path.exists(cond_dir):
            os.makedirs(cond_dir)
        wav_scp_file = open("{}/wav.scp".format(cond_dir), 'w')
        utt2spk_file = open("{}/utt2spk".format(cond_dir), 'w')
        for f in filelist:
            uttname = f.strip('.wav')
            wav_scp_file.write("{} {}/{}/{}\n".format(uttname, wav_dir, cond, f))
            utt2spk_file.write("{} {}\n".format(uttname, uttname))
        wav_scp_file.close()
        utt2spk_file.close()
        shutil.copyfile("{}/utt2spk".format(cond_dir), "{}/spk2utt".format(cond_dir))
    return 0

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*- #
"""*********************************************************************************************"""
#   FileName     [ ark2timit.py ]
#   Synopsis     [ process the .ark file preprocessed by kaldi for our dataloader ]
#   Author       [ Andy T. Liu (Andi611) ]
#   Copyright    [ Copyleft(c), Speech Lab, NTU, Taiwan ]
#   Reference    [ https://github.com/nttcslab-sp/kaldiio ]
"""*********************************************************************************************"""

###############
# IMPORTATION #
###############
import os
import pickle
import operator
import numpy as np
import pandas as pd
from tqdm import tqdm
from kaldiio import ReadHelper


############
# SETTINGS #
############
KALDI_ROOT = '/media/andi611/1TBSSD/kaldi/' # change this to your own kaldi root
TIMIT_PATH = os.path.join(KALDI_ROOT, 'egs/timit/s5/data-fmllr-tri3/') # this needs to be generated by kaldi scripts
OUTPUT_DIR = '../data/timit_fmllr_cmvn' # 

SETS = ['dev', 'test', 'train'] # you should not need to change this


########
# MAIN #
########
def main():
    if not os.path.isdir(KALDI_ROOT):
        print('CHANGE THIS TO YOUR OWN KALDI ROOT: ', KALDI_ROOT)
        exit()

    if not os.path.isdir(TIMIT_PATH):
        print('Invalid path for the kaldi TIMIT dataset: ', TIMIT_PATH)
        print('Please run the kaldi scripts first! More information are described in the README file and Wiki page.')

    if not os.path.isdir(OUTPUT_DIR):
        os.mkdir(OUTPUT_DIR)

    # read data from the preprocessed kaldi directory
    for s in SETS:
        
        output = {}
        print('Preprocessing', s, 'data...')
        cur_dir = os.path.join(OUTPUT_DIR, s.replace('_', '-'))
        if not os.path.isdir(cur_dir): os.mkdir(cur_dir)
        
        for i in range(10):
            with ReadHelper('ark:' + TIMIT_PATH + s + '/data/feats_fmllr_' + s + '.' + str(i+1) + '.ark') as reader:
                for key, array in tqdm(reader):
                    array = np.asarray(array).astype('float32')
                    np.save(os.path.join(cur_dir, key), array)
                    output[os.path.join(s.replace('_', '-'), key + '.npy')] = len(array)

        output = sorted(output.items(), key=operator.itemgetter(1), reverse=True)
        df = pd.DataFrame(data={'file_path':[fp for fp, l in output], 'length':[l for fp, l in output], 'label':'None'})
        df.to_csv(os.path.join(OUTPUT_DIR, s.replace('_', '-') + '.csv'))

    print('[ARK-TO-TIMIT] - All done, saved at \'' + str(OUTPUT_DIR) + '\', exit.')
    exit()

if __name__ == '__main__':
    main()
    



from collections import OrderedDict

from s3prl.dataset.common_pipes import EncodeMultiLabel
from s3prl.dataset.utterance_classification_pipe import HearScenePipe
from s3prl.util.pseudo_data import pseudo_audio


def test_multi_label():
    data = OrderedDict(
        {
            "0": dict(labels=["hello", "best", "because", "maybe"]),
            "1": dict(labels=["hello", "because", "cat"]),
            "2": dict(labels=["cat", "dog"]),
        }
    )
    dataset = EncodeMultiLabel()(data)
    category = dataset.get_tool("category")
    assert "binary_labels" in dataset[0]


def test_hear_scene():
    with pseudo_audio([8, 10, 3], sample_rate=16000) as (paths, lengths):
        data = OrderedDict(
            {
                "0": dict(
                    wav_path=paths[0], labels=["hello", "best", "because", "maybe"]
                ),
                "1": dict(wav_path=paths[1], labels=["hello", "because", "cat"]),
                "2": dict(wav_path=paths[2], labels=["cat", "dog"]),
            }
        )
        dataset = HearScenePipe()(data)
        item = dataset[0]

        assert "x" in item
        assert "x_len" in item
        assert "y" in item
        assert "labels" in item
        assert "unique_name" in item



# -*- coding: utf-8 -*-
# @Time    : 8/25/21 5:25 PM
# @Author  : Yuan Gong
# @Affiliation  : Massachusetts Institute of Technology
# @Email   : yuangong@mit.edu
# @File    : expert.py

import torch
import logging

from s3prl.upstream.interfaces import SAMPLE_RATE

from .audio import FeatureExtractor

logger = logging.getLogger(__name__)


class UpstreamExpert(torch.nn.Module):
    def __init__(self, **kwargs):
        super().__init__()

        try:
            import timm
            from .ast_models import ASTModel
        except:
            print(
                "SSAST requires 'timm==0.4.5' to work. Please run 'pip install timm==0.4.5'"
            )
            raise

        model_size = kwargs["model_size"]
        pretrain_path = kwargs["pretrain_path"]
        target_length = kwargs["target_length"]
        self.expected_secs = float(target_length / 100)

        model_size, model_type = model_size.split("_")[0], model_size.split("_")[1]
        self.preprocessor = FeatureExtractor(
            target_length=target_length, apply_cmvn=False
        )
        if model_type == "p":
            logger.info("now train a patch models")
            self.model = ASTModel(
                fshape=16,
                tshape=16,
                fstride=10,
                tstride=10,
                input_tdim=target_length,
                input_fdim=128,
                model_size=model_size,
                pretrain_stage=False,
                load_pretrained_mdl_path=pretrain_path,
            )
            self.vertical_num_patches = (128 - 16) // 10 + 1  # 12
        else:
            logger.info("now train a frame models")
            self.model = ASTModel(
                fshape=128,
                tshape=2,
                fstride=128,
                tstride=1,
                input_tdim=target_length,
                input_fdim=128,
                model_size=model_size,
                pretrain_stage=False,
                load_pretrained_mdl_path=pretrain_path,
            )
            self.vertical_num_patches = (128 - 128) // 128 + 1  # 1

    def get_downsample_rates(self, key: str = None) -> int:
        return 160

    def forward(self, wavs):
        wavs_len = [len(wav) for wav in wavs]
        max_wav_len = max(wavs_len)
        seg_n_sample = int(self.expected_secs * SAMPLE_RATE)
        padded_max_wav_len = (max_wav_len // seg_n_sample + 1) * seg_n_sample
        padded_wavs = [
            torch.cat([wav, wav.new_zeros(padded_max_wav_len - len(wav))])
            for wav in wavs
        ]

        all_hs = []
        for start in range(0, max(len(wav) for wav in padded_wavs), seg_n_sample):
            subwavs = [wav[start : start + seg_n_sample] for wav in padded_wavs]
            features = [self.preprocessor(wav.unsqueeze(0)) for wav in subwavs]
            features = torch.stack(features, dim=0)
            hidden_states, _ = self.model(features)
            all_hs.append(hidden_states)

        all_chunked_hs = list(zip(*all_hs))
        all_hs = []
        max_feat_len = max([l // self.get_downsample_rates() for l in wavs_len])
        for chunk_hs in all_chunked_hs:
            all_hs.append(torch.cat(chunk_hs, dim=1)[:, :max_feat_len, :])

        return {"hidden_states": all_hs}



import os
import pandas as pd
from pydub import AudioSegment
import sys

audio_path_root = sys.argv[1]

audio_path = os.path.join(audio_path_root, "Full/WAV_16000")
label_path = "./utility/CMU_MOSEI_Labels.csv"
out_path = os.path.join(audio_path_root, "Segmented_Audio")

if not os.path.exists(out_path):
    os.makedirs(out_path)
    os.makedirs(os.path.join(out_path, "train"))
    os.makedirs(os.path.join(out_path, "dev"))
    os.makedirs(os.path.join(out_path, "test"))

df = pd.read_csv(label_path)

for row in df.itertuples():
    unsegmented = AudioSegment.from_wav(os.path.join(audio_path, 
		row.file + ".wav"))
    segment = unsegmented[max(0, row.start * 1000) : row.end * 1000]
    if row.split == 0:
        segment.export(
            os.path.join(out_path, 
						 "train/" + row.file + "_" + str(row.index) + ".wav"),
            format="wav",
            bitrate="256k",
        )
    elif row.split == 1:
        segment.export(
            os.path.join(out_path, 
						 "dev/" + row.file + "_" + str(row.index) + ".wav"),
            format="wav",
            bitrate="256k",
        )
    elif row.split == 2:
        segment.export(
            os.path.join(out_path, 
						 "test/" + row.file + "_" + str(row.index) + ".wav"),
            format="wav",
            bitrate="256k",
        )
